import c4d
from cams_ids import *
from cams_utils_image import *
from cams_utils_pref import *
from cams_utils import *
from cams_dialog_cams import DialogMainCams
from cams_dialog_auth import Auth, TestAuth


g_dlgMainCams = None

# To be called during shutdown
def CmdCamsDestroyGlobals():
    global g_dlgMainCams

    if g_dlgMainCams is not None:
        if not g_dlgMainCams.Close():
            c4d.gui.MessageDialog('{0} SEVERE ISSUE: FAILED TO CLOSE DIALOG'.format(PLUGIN_NAME))

        g_dlgMainCams = None


def RegisterCommandCams():
    bmpIcon = LoadImage('icon', guiPixelRatio=2.0)

    pluginFlags = c4d.PLUGINFLAG_COMMAND_OPTION_DIALOG

    res = c4d.plugins.RegisterCommandPlugin(PLUGIN_ID_CAMS, PLUGIN_NAME,
                                            pluginFlags, bmpIcon, PLUGIN_TOOLTIP, CommandDataCams())
    if not res:
        PrintError('Failed to initialize CommandData ({0})'.format(PLUGIN_ID_CAMS))
        return False

    # TODO Andreas: Report to Maxon: Looks like Manager is not unregistered on Reload Python Plugins
    pluginManager = c4d.plugins.FindPlugin(PLUGIN_ID_CAMS_MANAGER, type=c4d.PLUGINTYPE_MANAGERINFORMATION)
    if pluginManager is None:
        c4d.plugins.RegisterManagerInformation(PLUGIN_ID_CAMS_MANAGER, '{0} Dialog'.format(PLUGIN_NAME), 0)

    return True


####################################################################################################
# Class CommandDataCams ############################################################################
####################################################################################################
class CommandDataCams(c4d.plugins.CommandData):
    _dlgMain = None


    def Execute(self, doc):
        if not Auth():
            return True

        global g_dlgMainCams

        bcInputState = c4d.BaseContainer()
        if c4d.gui.GetInputState(c4d.BFM_INPUT_KEYBOARD, c4d.BFM_INPUT, bcInputState):
            if bcInputState[c4d.BFM_INPUT_QUALIFIER] & 0x02: # TODO Andreas: c4d.QUALIFIER_CTRL??? c4d.QCTRL???
                bcMenu = c4d.BaseContainer()

                bcDoc = GetDocBC(doc)

                cameras = CameraBCToList(bcDoc)

                for idxCam, (objCam, _, _, _, isEditorCam) in enumerate(cameras):
                    if isEditorCam:
                        name = 'Editor Camera'

                        bcMenu.InsData(0, '')

                    else:
                        name = objCam.GetName()

                    bcMenu.InsData(c4d.FIRST_POPUP_ID + idxCam, name)

                result = c4d.gui.ShowPopupDialog(cd=None, bc=bcMenu, x=c4d.MOUSEPOS, y=c4d.MOUSEPOS)
                if result == 0:
                    return True

                idxCam = result - c4d.FIRST_POPUP_ID

                objCamActive = cameras[idxCam][0]

                bdActive = doc.GetRenderBaseDraw()
                bdActive.SetSceneCamera(objCamActive)

                c4d.EventAdd()

                return True

        if self._dlgMain is None:
            self._dlgMain = DialogMainCams()
            g_dlgMainCams = self._dlgMain

        if self._dlgMain is None:
            PrintError('Unable to allocate main dialog.')
            return True

        if c4d.GetC4DVersion() // 1000 >= 25:
            if self._dlgMain.IsOpen() and not self._dlgMain.GetFolding():
                self._dlgMain.SetFolding(True)
                return True

        else:
            if self._dlgMain.IsOpen():
                self._dlgMain.Close()
                return True

        self._dlgMain.Open(c4d.DLG_TYPE_ASYNC, PLUGIN_ID_CAMS, xpos=-1, ypos=-1, defaultw=500, defaulth=0) # pos: -1: mouse pos, -2: screen center

        return True


    def ExecuteOptionID(self, doc, plugid, subid):
        if not Auth():
            return True

        # Workaround for "Options being called from Extensions menu"
        # https://plugincafe.maxon.net/topic/12894/different-behavior-on-mac-for-commanddata-options
        # https://plugincafe.maxon.net/topic/11837/r21-commanddata-with-option-not-working-on-macos
        if c4d.GeGetCurrentOS() == c4d.OPERATINGSYSTEM_OSX and c4d.GetC4DVersion() // 1000 >= 21:
            if subid == 0:
                c4d.CallCommand(PLUGIN_ID_CAMS)
                return True

        if c4d.GetC4DVersion() // 1000 >= 19:
            c4d.PrefsLib_OpenDialog(PLUGIN_ID_CAMS_PREFS)

        else:
            PrintError('Cinema 4D below R19 not supported.')

        return True


    def RestoreLayout(self, secref):
        if not TestAuth():
            return True

        if self._dlgMain is None:
            global g_dlgMainCams

            self._dlgMain = DialogMainCams()
            g_dlgMainCams = self._dlgMain

        return self._dlgMain.Restore(PLUGIN_ID_CAMS, secref)


    def GetState(self, doc):
        if not ReadPrefValue(ID_PREF_PLUGIN_ENABLED):
            return 0

        state = c4d.CMD_ENABLED

        if self._dlgMain is None:
            return state

        isFolded = False
        if c4d.GetC4DVersion() // 1000 >= 25:
            isFolded = self._dlgMain.GetFolding()

        if self._dlgMain.IsOpen() and not isFolded:
            state |= c4d.CMD_VALUE

        return state
