import math
import c4d
from cams_ids import *
from cams_utils_image import *
from cams_utils_pref import *
from cams_utils import *
from cams_dialog_auth import Auth
from cams_tag_preview_cam import TagPreviewCamera


def RegisterTagPreviewCamera(res):
    global __res__

    __res__ = res

    # Note: The tag gets registered with alternative icon (Look Around cam).
    #       The actual icon of the tag is determined via MSG_GETCUSTOMICON.
    #       This is done, in order to have both icons (Turntable Cam icon for Command Plugin and Look Around Cam icon for tag)
    #       available without registering icons with extra plugin IDs.
    bmpIcon = LoadImage('cam_Lookaround', guiPixelRatio=2.0)

    res = c4d.plugins.RegisterTagPlugin(PLUGIN_ID_CAMS_TAG_PREVIEW_CAMERA, PLUGIN_NAME_TAG_PREVIEW_CAMERA,
                                        c4d.TAG_EXPRESSION | c4d.TAG_VISIBLE, g=TagPreviewCamera,
                                        description="Tcamspreviewcamera", icon=bmpIcon)
    if not res:
        PrintError('Failed to initialize TagData ({0}, {1})'.format(PLUGIN_NAME_TAG_PREVIEW_CAMERA, PLUGIN_ID_CAMS_TAG_PREVIEW_CAMERA))
        return False

    bmpIcon = LoadImage('cam_Turnaround', guiPixelRatio=2.0)

    res = c4d.plugins.RegisterCommandPlugin(PLUGIN_ID_CAMS_COMMAND_PREVIEW_CAMERA, PLUGIN_NAME_TAG_PREVIEW_CAMERA,
                                            0, bmpIcon, PLUGIN_TOOLTIP_TAG_PREVIEW_CAMERA, CommandDataCreatePreviewCam())
    if not res:
        PrintError('Failed to initialize CommandData ({0}, {1})'.format(PLUGIN_NAME_TAG_PREVIEW_CAMERA, PLUGIN_ID_CAMS_COMMAND_PREVIEW_CAMERA))
        return False

    return True


####################################################################################################
# Class CommandDataCreatePreviewCam ################################################################
####################################################################################################
class CommandDataCreatePreviewCam(c4d.plugins.CommandData):

    def ExecuteCreateTurntableCam(self, doc, activeIsMultiple, objActive, objCameraNew, tagCamera):
        doc.InsertObject(objCameraNew)
        doc.AddUndo(c4d.UNDOTYPE_NEW, objCameraNew)

        objCameraNew.SetName('Turntable Camera')

        tagCamera[c4d.CAMS_PREVIEW_CAMERA_MODE] = c4d.CAMS_PREVIEW_CAMERA_MODE_OUTSIDE # actually default anyway

        if activeIsMultiple or objActive is None:
            objTarget = c4d.BaseObject(c4d.Onull)
            objTarget.SetName('Camera Target')
            if c4d.GetC4DVersion() // 1000 > 20:
                objTarget[c4d.ID_BASELIST_ICON_COLORIZE_MODE] = 1
                objTarget[c4d.ID_BASELIST_ICON_COLOR] = c4d.Vector(1.0, 0.9, 0.4)

            if objActive is not None:
                objTarget.SetMg(objActive.GetMg())

            doc.InsertObject(objTarget)
            doc.AddUndo(c4d.UNDOTYPE_NEW, objTarget)

        else:
            objTarget = objActive

        tagCamera[c4d.CAMS_PREVIEW_CAMERA_TARGET] = objTarget

        objHeightHelper = c4d.BaseObject(c4d.Onull)
        objHeightHelper.SetName('Camera Height Target')
        if c4d.GetC4DVersion() // 1000 > 20:
            objHeightHelper[c4d.ID_BASELIST_ICON_COLORIZE_MODE] = 1
            objHeightHelper[c4d.ID_BASELIST_ICON_COLOR] = c4d.Vector(1.0, 0.9, 0.4)

        tagProtection = objHeightHelper.MakeTag(c4d.Tprotection)
        tagProtection[c4d.PROTECTION_P_Y] = False

        objHeightHelper.InsertUnder(objTarget)
        doc.AddUndo(c4d.UNDOTYPE_NEW, objHeightHelper)

        objTarget.ChangeNBit(c4d.NBIT_OM1_FOLD, c4d.NBITCONTROL_SET)
        objTarget.ChangeNBit(c4d.NBIT_OM2_FOLD, c4d.NBITCONTROL_SET)
        objTarget.ChangeNBit(c4d.NBIT_OM3_FOLD, c4d.NBITCONTROL_SET)
        objTarget.ChangeNBit(c4d.NBIT_OM4_FOLD, c4d.NBITCONTROL_SET)

        tagCamera[c4d.CAMS_PREVIEW_CAMERA_HEIGHT_HELPER] = objHeightHelper

        objHeightHelper.SetMg(objTarget.GetMg())

        return True


    def ExecuteCreateLookAroundCam(self, doc, activeIsMultiple, objActive, objCameraNew, tagCamera):
        objCameraNew.SetName('Look Around Camera')

        objCameraNew.SetAbsPos(c4d.Vector(0.0))
        objCameraNew.SetAbsRot(c4d.Vector(0.0))

        tagCamera[c4d.CAMS_PREVIEW_CAMERA_MODE] = c4d.CAMS_PREVIEW_CAMERA_MODE_INSIDE

        objPivot = c4d.BaseObject(c4d.Onull)
        objPivot.SetName('Look Around Camera Pivot')
        if c4d.GetC4DVersion() // 1000 > 20:
            objPivot[c4d.ID_BASELIST_ICON_COLORIZE_MODE] = 1
            objPivot[c4d.ID_BASELIST_ICON_COLOR] = c4d.Vector(1.0, 0.9, 0.4)

        if objActive is not None:
            if activeIsMultiple:
                mgPivot = objPivot.GetMg()

                mgPivot.off = objActive.GetMg().off

                objPivot.SetMg(mgPivot)

                tagCamera[c4d.CAMS_PREVIEW_CAMERA_RADIUS] = 0.0

                objParent = objPivot

            else:
                bbRad = objActive.GetRad()

                tagCamera[c4d.CAMS_PREVIEW_CAMERA_RADIUS] = math.sqrt(bbRad.x * bbRad.x + bbRad.z * bbRad.z) + 1.0

                objParent = objActive

        else:
            tagCamera[c4d.CAMS_PREVIEW_CAMERA_RADIUS] = 0.0

            objParent = objPivot

        objCameraNew.InsertUnder(objParent)

        if objParent == objPivot:
            doc.InsertObject(objPivot)
            doc.AddUndo(c4d.UNDOTYPE_NEW, objPivot)

            objPivot.ChangeNBit(c4d.NBIT_OM1_FOLD, c4d.NBITCONTROL_SET)
            objPivot.ChangeNBit(c4d.NBIT_OM2_FOLD, c4d.NBITCONTROL_SET)
            objPivot.ChangeNBit(c4d.NBIT_OM3_FOLD, c4d.NBITCONTROL_SET)
            objPivot.ChangeNBit(c4d.NBIT_OM4_FOLD, c4d.NBITCONTROL_SET)

        else:
            doc.AddUndo(c4d.UNDOTYPE_NEW, objCameraNew)

        return True


    def ExecuteSubID(self, doc, subid):
        if not Auth():
            return True

        objCamActive, isEditorCam = GetActiveCamera(doc)

        objCameraNew = objCamActive.GetClone()
        if objCameraNew is None:
            return True

        if isEditorCam:
            objCameraNew[c4d.CAMERAOBJECT_STEREO_MODE] = 0

        tags = GetAllCameraTags(objCameraNew)
        for tag in tags:
            tag.Remove()

        createRendererCam = ReadPrefValue(ID_PREF_CREATE_CAMERAS)

        tagRenderer = None

        if createRendererCam == 1: # Redshift
            tagRenderer = c4d.BaseTag(1036760)

        elif createRendererCam == 2: # Corona
            tagRenderer = c4d.BaseTag(1032177)

        elif createRendererCam == 3: # Ocatne
            tagRenderer = c4d.BaseTag(1029524)

        elif createRendererCam == 4: # Vray
            tagRenderer = c4d.BaseTag(1053283)

        if tagRenderer is not None:
            objCameraNew.InsertTag(tagRenderer)

        realActive = doc.GetRealActiveObject()

        if realActive is None:
            objActive, activeIsMultiple = None, False

        else:
            objActive, activeIsMultiple = realActive

        tagCamera = c4d.BaseTag(PLUGIN_ID_CAMS_TAG_PREVIEW_CAMERA)
        if tagCamera is None:
            return True

        objCameraNew.InsertTag(tagCamera)

        doc.StartUndo()

        if subid == COMMAND_PREVIEW_CAMERA_SUBID_TURNTABLE:
            self.ExecuteCreateTurntableCam(doc, activeIsMultiple, objActive, objCameraNew, tagCamera)

        elif subid == COMMAND_PREVIEW_CAMERA_SUBID_LOOK_AROUND:
            self.ExecuteCreateLookAroundCam(doc, activeIsMultiple, objActive, objCameraNew, tagCamera)

        else:
            PrintError('Unknown Sub-ID ({0})'.format(subid))

        doc.SetActiveObject(objCameraNew)
        doc.SetActiveTag(tagCamera)
        c4d.gui.ActiveObjectManager_SetObject(c4d.ACTIVEOBJECTMODE_TAG, tagCamera, c4d.ACTIVEOBJECTMANAGER_SETOBJECTS_OPEN, activepage=c4d.DescID())

        doc.EndUndo()

        c4d.EventAdd()

        return True


    def GetSubContainer(self, doc, submenu):
        bc = c4d.BaseContainer()

        submenu.InsData(COMMAND_PREVIEW_CAMERA_SUBID_TURNTABLE, '{0} Turntable Camera&i{1}&'.format(PLUGIN_NAME, PLUGIN_ID_CAMS_COMMAND_PREVIEW_CAMERA))
        submenu.InsData(COMMAND_PREVIEW_CAMERA_SUBID_LOOK_AROUND, '{0} Look Around Camera&i{1}&'.format(PLUGIN_NAME, PLUGIN_ID_CAMS_TAG_PREVIEW_CAMERA))

        return True
