import c4d
from cams_ids import *
from cams_ids_dialogs import *
from cams_utils_image import *
from cams_utils_pref import *
from cams_utils import *
from cams_dialog_popup_command_panel import PopupCommandPanelIcons, DialogPopupCommandPanel
from cams_dialog_popup_per_camera import PopupPerCameraIcons, DialogPopupPerCamera


####################################################################################################
# Class DialogInPaletteComboCamSwitcher ############################################################
####################################################################################################
ID_COMBO_CAMS            = 1000
ID_BUTTON_CURRENT_CAM    = 1001
ID_BUTTON_ALL_CAMS_POPUP = 1002


class DialogInPaletteComboCamSwitcher(c4d.gui.GeDialog):
    _bitmapButtonCommandPanel = None
    _bmpCommandPanel = None
    _bmpCommandPanelRenderBusy = None

    _bmpOptions = None

    _iconsPopupCommandPanel = None
    _dlgPopupCommandPanel = None

    _iconsPopupPerCamera = None
    _dlgPopupPerCamera = None


    def __init__(self):
        self._iconsPopupCommandPanel = PopupCommandPanelIcons()
        self._dlgPopupCommandPanel = None

        self._iconsPopupPerCamera = PopupPerCameraIcons()
        self._dlgPopupPerCamera = None

        self._bitmapButtonCommandPanel = None
        self.InitCommandPanelIcons()

        self.AddGadget(c4d.DIALOG_NOMENUBAR, 0)

        c4d.gui.GeDialog.__init__(self)


    def InitCommandPanelIcons(self):
        sizePalette = ReadPrefValue(ID_PREF_IN_PALETTE_SIZE)
        if sizePalette == 0:
            sizeFactor = 0.5

        elif sizePalette == 1:
            sizeFactor = 0.75

        elif sizePalette == 2:
            sizeFactor = 1.0

        self._bmpCommandPanel = LoadImage('icon', guiPixelRatio=2.0, factor=sizeFactor)
        self._bmpCommandPanelRenderBusy = LoadImage('icon_render_busy', guiPixelRatio=2.0, factor=sizeFactor)
        self._bmpOptions = GetIconImage(ID_ICON_CURRENT_CAM, factor=sizeFactor)

    # Only cleanup in here
    def CoreMessageCommandPanelPopupCommand(self):
        if self._dlgPopupCommandPanel is None:
            return

        if self._dlgPopupCommandPanel.IsOpen():
            PrintWarn('Popup dialog still open?!?!?!')
            self._dlgPopupCommandPanel.Close()

        self._dlgPopupCommandPanel = None


    # Only cleanup in here
    def CoreMessagePerCameraPopupCommand(self):
        if self._dlgPopupPerCamera is None:
            return

        if self._dlgPopupPerCamera.IsOpen():
            PrintWarn('Popup dialog still open?!?!?!')
            self._dlgPopupPerCamera.Close()

        self._dlgPopupPerCamera = None


    def CommandComboCams(self):
        idxCam = self.GetInt32(ID_COMBO_CAMS)

        doc = c4d.documents.GetActiveDocument()

        bcDoc = GetDocBC(doc)

        cameras = CameraBCToList(bcDoc)

        objCamActive = cameras[idxCam][0]

        bdActive = doc.GetRenderBaseDraw()
        bdActive.SetSceneCamera(objCamActive)

        c4d.EventAdd()


    def CommandCommandPanelPopup(self, x=-1, y=-1):
        if self._dlgPopupCommandPanel is not None:
            self._dlgPopupCommandPanel.Close()
            self._dlgPopupCommandPanel = None

        self._dlgPopupCommandPanel = DialogPopupCommandPanel(self._iconsPopupCommandPanel, self._deferredRenderBusy)

        x, y = AtMousePointer(self, x, y)

        self._dlgPopupCommandPanel.Open(dlgtype=c4d.DLG_TYPE_ASYNC_POPUPEDIT, pluginid=0, xpos=x, ypos=y, defaultw=0, defaulth=0, subid=0)

        return


    def CommandPerCameraPopup(self, x=-1, y=-1):
        doc = c4d.documents.GetActiveDocument()

        bcDoc = GetDocBC(doc)

        cameras = CameraBCToList(bcDoc)

        objCam, isEditorCam = GetActiveCamera(doc)
        idxCam = GetCameraIndex(cameras, objCam)

        if self._dlgPopupPerCamera is not None:
            self._dlgPopupPerCamera.Close()
            self._dlgPopupPerCamera = None

        self._dlgPopupPerCamera = DialogPopupPerCamera(self._iconsPopupPerCamera, idxCam, objCam, isEditorCam, None, self._deferredRenderBusy)

        x, y = AtMousePointer(self, x, y)

        self._dlgPopupPerCamera.Open(dlgtype=c4d.DLG_TYPE_ASYNC_POPUPEDIT, pluginid=0, xpos=x, ypos=y, defaultw=0, defaulth=0, subid=0)

        return


    def CreateLayout(self):
        if self.GroupBegin(42, c4d.BFH_LEFT | c4d.BFV_BOTTOM, cols=4, initw=200, inith=1):
            self.GroupBorderSpace(5, 0, 3, 0)

            self._bitmapButtonCommandPanel = CreateLayoutAddBitmapButton(self, ID_BUTTON_ALL_CAMS_POPUP, bmp1=self._bmpCommandPanel, \
                                                                         tooltip='{0} Command Panel'.format(PLUGIN_NAME), button=True, toggle=False, flags=c4d.BFH_LEFT | c4d.BFV_CENTER, inMenu=False)
            self.AddComboBox(ID_COMBO_CAMS, c4d.BFH_LEFT | c4d.BFV_SCALEFIT, initw=200)
            CreateLayoutAddBitmapButton(self, ID_BUTTON_CURRENT_CAM, bmp1=self._bmpOptions, \
                                        tooltip='Current Cam Options', button=True, toggle=False, flags=c4d.BFH_LEFT | c4d.BFV_CENTER, inMenu=False)

        self.GroupEnd()

        return True


    def InitValues(self):
        doc = c4d.documents.GetActiveDocument()

        self.FreeChildren(ID_COMBO_CAMS)

        bcDoc = GetDocBC(doc)

        cameras = CameraBCToList(bcDoc)

        objCamActive, isEditorCam = GetActiveCamera(doc)

        for idxCam, (objCam, _, _, _, isEditorCam) in enumerate(cameras):
            if isEditorCam:
                name = 'Editor Camera'

                self.AddChild(ID_COMBO_CAMS, -1, '')

            else:
                name = objCam.GetName()

            self.AddChild(ID_COMBO_CAMS, idxCam, name)

            if objCam == objCamActive:
                self.SetInt32(ID_COMBO_CAMS, idxCam)

        self.LayoutChanged(42)

        return True


    _deferredRenderBusy = False
    _iconBusyState = False
    def Timer(self, msg): # delayed scrollgroup adjust
        self._iconBusyState = not self._iconBusyState
        if self._iconBusyState:
            self._bitmapButtonCommandPanel.SetImage(self._bmpCommandPanelRenderBusy)

        else:
            self._bitmapButtonCommandPanel.SetImage(self._bmpCommandPanel)


    def CoreMessage(self, id, msg):
        if id != PLUGIN_ID_CAMS_COREMESSAGE:
            return c4d.gui.GeDialog.CoreMessage(self, id, msg)

        subIdCoreMessage = GetCoreMessageParam(msg, c4d.BFM_CORE_PAR1)

        if subIdCoreMessage == SUBID_COREMESSAGE_CAMERA_LIST_CHANGE or \
           subIdCoreMessage == SUBID_COREMESSAGE_CAMERA_ACTIVE_CHANGE:
            self.InitValues()

        elif subIdCoreMessage >= SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE and subIdCoreMessage < SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE + 1000000:
            self.CoreMessageCommandPanelPopupCommand()

        elif subIdCoreMessage >= SUBID_COREMESSAGE_DLG_PER_CAM_BASE and subIdCoreMessage < SUBID_COREMESSAGE_DLG_PER_CAM_BASE + 1000000:
            self.CoreMessagePerCameraPopupCommand()

        elif subIdCoreMessage == SUBID_COREMESSAGE_DEFERRED_RENDER_BUSY:
            self._deferredRenderBusy = True
            self.SetTimer(250)

        elif subIdCoreMessage == SUBID_COREMESSAGE_DEFERRED_RENDER_IDLE:
            self._deferredRenderBusy = False
            self.SetTimer(0)
            self._bitmapButtonCommandPanel.SetImage(self._bmpCommandPanel)
            self._iconBusyState = False


        return c4d.gui.GeDialog.CoreMessage(self, id, msg)


    def Command(self, id, msg):
        if id == ID_COMBO_CAMS:
            self.CommandComboCams()

        elif id == ID_BUTTON_ALL_CAMS_POPUP:
            self.CommandCommandPanelPopup()

        elif id == ID_BUTTON_CURRENT_CAM:
            self.CommandPerCameraPopup()

        return True


    def GetResult(self):
        return self._result
