import c4d
from cams_utils_dialog import *
from cams_ids import *
from cams_ids_dialogs import *
from cams_utils_image import *
from cams_utils import *


####################################################################################################
# Class DialogPopupRenderAll #######################################################################
####################################################################################################
class PopupCommandPanelIcons():

    def __init__(self):
        self._bmpIconRender = LoadImage('render_all_cams_default', guiPixelRatio=2.0, factor=1.5)  ##### cmd_render_cam
        self._bmpIconRenderVP = LoadImage('render_all_cams_viewport', guiPixelRatio=2.0, factor=1.5)
        self._bmpIconRenderCurrentRD = LoadImage('render_all_cams_render', guiPixelRatio=2.0, factor=1.5)

        self._bmpIconProofSheet = LoadImage('proof_sheet_default', guiPixelRatio=2.0, factor=1.5)
        self._bmpIconProofSheetVP = LoadImage('proof_sheet_viewport', guiPixelRatio=2.0, factor=1.5)
        self._bmpIconProofSheetCurrentRD = LoadImage('proof_sheet_render', guiPixelRatio=2.0, factor=1.5)

        self._bmpIconVisDefault = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT3, factor=2.0)
        self._bmpIconVisOn = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT1, factor=2.0)
        self._bmpIconVisOff = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT2, factor=2.0)

        self._bmpIconStandardCamera = GetIconImage(ID_ICON_CAMERA_STANDARD, factor=1.0)
        self._bmpIconTargetCamera = GetIconImage(ID_ICON_CAMERA_TARGET, factor=1.0)
        self._bmpIconMotionCamera = GetIconImage(ID_ICON_CAMERA_MOTION, factor=1.0)
        self._bmpIconStereoCamera = GetIconImage(ID_ICON_CAMERA_STEREO, factor=1.0)
        self._bmpIconCameraCrane = GetIconImage(ID_ICON_CAMERA_CRANE, factor=1.0)

        self._bmpIconTurntableCamera = LoadImage('cam_Turnaround', guiPixelRatio=2.0, factor=1.0)
        self._bmpIconLookAroundCamera = LoadImage('cam_Lookaround', guiPixelRatio=2.0, factor=1.0)

        if HaveRedshift():
            tagRedshiftCamera = c4d.BaseTag(1036760)
            self._bmpIconRSStandardCamera = GetIconImage(tagRedshiftCamera)

        if HaveCorona():
            tagCoronaCamera = c4d.BaseTag(1032177)
            self._bmpIconCOCamera = GetIconImage(tagCoronaCamera)

        if HaveOctane():
            tagOctaneCamera = c4d.BaseTag(1029524)
            self._bmpIconOCCamera = GetIconImage(tagOctaneCamera)

        if HaveVray():
            tagVrayCamera = c4d.BaseTag(1053283)
            self._bmpIconVRCamera = GetIconImage(tagVrayCamera, factor=0.5)


class DialogPopupCommandPanel(c4d.gui.GeDialog):

    _icons = None

    _deferredRenderBusy = False

    _w = 0


    def __init__(self, icons, deferredRenderBusy, initw=0):
        self._icons = icons

        self._deferredRenderBusy = deferredRenderBusy

        self._w = initw

        c4d.gui.GeDialog.__init__(self)


    def CreateLayout(self):
        if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, initw=self._w, cols=1):
            self.GroupBorderSpace(5, 5, 5, 7)
            self.GroupBorderNoTitle(c4d.BORDER_ROUND)
            self.GroupSpace(0, 0)

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Title
                self.GroupBorderSpace(0, 0, 0, 3)

                self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name='Render All Cams', borderstyle=c4d.BORDER_WITH_TITLE_BOLD)

            self.GroupEnd() # Title

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Render All Cameras
                self.GroupBorderSpace(0, 3, 0, 0)

                CreateLayoutAddBitmapButton(self, ID_RENDER_ALL_TO_PV, bmp1=self._icons._bmpIconRender, tooltip='All Cams as Separate Images', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                self.AddSeparatorH(0, flags=c4d.BFV_SCALEFIT)

                CreateLayoutAddBitmapButton(self, ID_RENDER_ALL_TO_PV_VP, bmp1=self._icons._bmpIconRenderVP, tooltip='All Cams as Separate Images\n(Viewport Render)', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_RENDER_ALL_TO_PV_CURRENT_RD, bmp1=self._icons._bmpIconRenderCurrentRD, tooltip='All Cams as Separate Images\n(Current Settings)', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

            self.GroupEnd() # Render All Cameras

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Render Proof Sheet
                self.GroupBorderSpace(0, 3, 0, 0)

                CreateLayoutAddBitmapButton(self, ID_RENDER_PROOF_SHEET, bmp1=self._icons._bmpIconProofSheet, tooltip='Render Proof Sheet', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                self.AddSeparatorH(0, flags=c4d.BFV_SCALEFIT)

                CreateLayoutAddBitmapButton(self, ID_RENDER_PROOF_SHEET_VP, bmp1=self._icons._bmpIconProofSheetVP, tooltip='Render Proof Sheet\n(Viewport Render)', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_RENDER_PROOF_SHEET_CURRENT_RD, bmp1=self._icons._bmpIconProofSheetCurrentRD, tooltip='Render Proof Sheet\n(Current Settings)', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

            self.GroupEnd() # Render Proof Sheet

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=1): # Separator
                self.GroupBorderSpace(0, 4, 0, 4)

                self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

            self.GroupEnd() # Separator

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Title Visibility
                self.GroupBorderSpace(0, 5, 0, 10)

                self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name='Global Visibilty', borderstyle=c4d.BORDER_WITH_TITLE_BOLD)

            self.GroupEnd() # Title Visibility

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, cols=2): # Visibility columns

                self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name='Editor')

                if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_CENTER, rows=1): # Editor Visibility
                    CreateLayoutAddBitmapButton(self, ID_EDITOR_VISIBILITY_ALL_ON, bmp1=self._icons._bmpIconVisOn, tooltip='Editor Visibility: On', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                    CreateLayoutAddBitmapButton(self, ID_EDITOR_VISIBILITY_ALL_OFF, bmp1=self._icons._bmpIconVisOff, tooltip='Editor Visibility: Off', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                    CreateLayoutAddBitmapButton(self, ID_EDITOR_VISIBILITY_ALL_DEFAULT, bmp1=self._icons._bmpIconVisDefault, tooltip='Editor Visibility: Default', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                self.GroupEnd() # Editor Visibility

                self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name='Render')

                if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_CENTER, rows=1): # Render Visibility
                    CreateLayoutAddBitmapButton(self, ID_RENDER_VISIBILITY_ALL_ON, bmp1=self._icons._bmpIconVisOn, tooltip='Render Visibility: On', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                    CreateLayoutAddBitmapButton(self, ID_RENDER_VISIBILITY_ALL_OFF, bmp1=self._icons._bmpIconVisOff, tooltip='Render Visibility: Off', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                    CreateLayoutAddBitmapButton(self, ID_RENDER_VISIBILITY_ALL_DEFAULT, bmp1=self._icons._bmpIconVisDefault, tooltip='Render Visibility: Default', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                self.GroupEnd() # Render Visibility

            self.GroupEnd() # Visibility columns

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=1): # Separator
                self.GroupBorderSpace(0, 4, 0, 4)

                self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

            self.GroupEnd() # Separator

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Title "Create Camera"
                self.GroupBorderSpace(0, 5, 0, 10)

                self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name='Create Camera', borderstyle=c4d.BORDER_WITH_TITLE_BOLD)

            self.GroupEnd() # Title "Create Camera"

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_CENTER, rows=1): # Standard Cameras
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_STANDARD, bmp1=self._icons._bmpIconStandardCamera, tooltip='Standard Camera', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_TARGET, bmp1=self._icons._bmpIconTargetCamera, tooltip='Target Camera', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_MOTION, bmp1=self._icons._bmpIconMotionCamera, tooltip='Motion Camera', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_STEREO, bmp1=self._icons._bmpIconStereoCamera, tooltip='Stereo Camera', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_CRANE, bmp1=self._icons._bmpIconCameraCrane, tooltip='Camera Crane', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

            self.GroupEnd() # Standard Cameras

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=1): # Separator
                self.GroupBorderSpace(0, 4, 0, 4)

                self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

            self.GroupEnd() # Separator

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_CENTER, rows=1): # C4D Cams Cameras
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_PREVIEW, bmp1=self._icons._bmpIconTurntableCamera, tooltip='Turntable Camera', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_PREVIEW_INSIDEOUT, bmp1=self._icons._bmpIconLookAroundCamera, tooltip='Look Around Camera', button=True, toggle=False,
                                            flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

            self.GroupEnd() # C4D Cams Cameras

            if HaveRedshift() or HaveCorona() or HaveOctane() or HaveVray():
                if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=1): # Separator
                    self.GroupBorderSpace(0, 4, 0, 4)

                    self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

                self.GroupEnd() # Separator

                if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_CENTER, cols=5): # 3rd party renderer Cameras
                    if HaveRedshift():
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_RS_STANDARD, bmp1=self._icons._bmpIconRSStandardCamera, tooltip='Redshift Standard Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_RS_FISHEYE, bmp1=self._icons._bmpIconRSStandardCamera, tooltip='Redshift Fisheye Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_RS_SPHERICAL, bmp1=self._icons._bmpIconRSStandardCamera, tooltip='Redshift Spherical Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_RS_CYLINDRICAL, bmp1=self._icons._bmpIconRSStandardCamera, tooltip='Redshift Cylindrical Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_RS_STEREO_SPHERICAL, bmp1=self._icons._bmpIconRSStandardCamera, tooltip='Redshift Sterao-Spherical Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                    if HaveCorona():
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_CO_STANDARD, bmp1=self._icons._bmpIconCOCamera, tooltip='Corona Standard Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                    if HaveOctane():
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_CO_STANDARD, bmp1=self._icons._bmpIconOCCamera, tooltip='Octane Standard Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                    if HaveVray():
                        CreateLayoutAddBitmapButton(self, ID_CREATE_CAMERA_VR_PHYSICAL, bmp1=self._icons._bmpIconVRCamera, tooltip='Vray Physical Camera', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                self.GroupEnd()  # 3rd party renderer Cameras

        self.GroupEnd()

        return True


    def CoreMessage(self, id, msg):
        if id != PLUGIN_ID_CAMS_COREMESSAGE:
            return c4d.gui.GeDialog.CoreMessage(self, id, msg)

        subIdCoreMessage = GetCoreMessageParam(msg, c4d.BFM_CORE_PAR1)

        if subIdCoreMessage == SUBID_COREMESSAGE_DEFERRED_RENDER_IDLE:
            self._deferredRenderBusy = False

        return c4d.gui.GeDialog.CoreMessage(self, id, msg)


    _dlgRequesterOpen = False

    def Message(self, msg, result):
        idMsg = msg.GetId()

        if idMsg == c4d.BFM_ACTION:
            if msg[c4d.BFM_ACTION_ID] == 1000 and msg[c4d.BFM_ACTION_INDRAG]:
                return True # avoid ending up in Command() during drag

        elif idMsg == 1649165891 and not self._dlgRequesterOpen: # TODO Andreas: message?
            #print('1649165891 lost focus?')
            self.Close()

        return c4d.gui.GeDialog.Message(self, msg, result)


    def Command(self, id, msg):
        if id == ID_POPUP_MENU__MUTE_BUTTON:
            return True # mute icon button

        elif id >= ID_EDITOR_VISIBILITY_ALL_DEFAULT and id <= ID_RENDER_VISIBILITY_ALL_OFF:
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE_KEEP + id)
            return True # keep open

        elif id == ID_RENDER_ALL_TO_PV or id == ID_RENDER_ALL_TO_PV_VP or id == ID_RENDER_ALL_TO_PV_CURRENT_RD or \
             id == ID_RENDER_PROOF_SHEET or id == ID_RENDER_PROOF_SHEET_VP or id == ID_RENDER_PROOF_SHEET_CURRENT_RD:
            self._dlgRequesterOpen = True # Block Message() closing dialog due to lost focus
            ShowRenderInBackgroundInfo()

            if self._deferredRenderBusy:
                result = ShowRenderInProgressQuestion()
                if result == 1:
                    c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_DEFERRED_RENDER_FLUSH_QUEUE)

                elif result == 2:
                    self._dlgRequesterOpen = False
                    return True

            self._dlgRequesterOpen = False

        self.Close()

        c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE + id)

        return True
