import c4d
from cams_utils_dialog import *
from cams_ids import *
from cams_ids_dialogs import *
from cams_utils_pref import *


####################################################################################################
# Class DialogPopupDesign ##########################################################################
####################################################################################################
class PopupDesignIcons():
    def __init__(self):
        self._bmpsIconThumbSize = []

        if c4d.GetC4DVersion() // 1000 >= 25:
            self._bmpsIconThumbSize.append(GetIconImage(16740, factor=0.5)) # Inherit Small Icons # TODO Andreas: symbol?
            self._bmpsIconThumbSize.append(GetIconImage(16741, factor=0.5)) # Inherit Medium Icons # TODO Andreas: symbol?
            self._bmpsIconThumbSize.append(GetIconImage(16742, factor=0.5)) # Inherit Large Icons # TODO Andreas: symbol?

        elif c4d.GetC4DVersion() // 1000 >= 20:
            self._bmpsIconThumbSize.append(GetIconImage(c4d.RESOURCEIMAGE_BROWSER_SMALLVIEW))
            self._bmpsIconThumbSize.append(GetIconImage(c4d.RESOURCEIMAGE_TIMELINE_STATE5))
            self._bmpsIconThumbSize.append(GetIconImage(c4d.RESOURCEIMAGE_BROWSER_BIGVIEW))

        else:
            self._bmpsIconThumbSize.append(GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT3))
            self._bmpsIconThumbSize.append(GetIconImage(c4d.RESOURCEIMAGE_TIMELINE_STATE5))
            self._bmpsIconThumbSize.append(GetIconImage(c4d.RESOURCEIMAGE_KEYFRAME_BUTTON_UP))

        self._bmpsIconButtonAttributes = []
        self._bmpsIconButtonAttributes.append(GetIconImage(ID_ICON_ATTRIBUTES_SHOW_BOTTOM))
        self._bmpsIconButtonAttributes.append(GetIconImage(ID_ICON_ATTRIBUTES_SHOW_RIGHT))
        self._bmpsIconButtonAttributes.append(GetIconImage(ID_ICON_ATTRIBUTES_DONT_SHOW))


class DialogPopupDesign(c4d.gui.GeDialog):

    _icons = None

    _w = 0

    _result = 0

    def __init__(self, icons, initw=250):
        self._icons = icons

        self._w = initw

        self._bitmapButtonEnableCombo = None
        self._bitmapButtonEnableTabs = None
        self._bitmapButtonEnableThumbs = None
        self._bitmapButtonThumbSize = None
        self._bitmapButtonAttributes = None

        self._result = 0

        c4d.gui.GeDialog.__init__(self)


    def CreateLayout(self):
        if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, initw=self._w, cols=1):
            self.GroupBorderSpace(5, 5, 5, 5)
            self.GroupBorderNoTitle(c4d.BORDER_ROUND)
            self.GroupSpace(0, 0)

            self._bitmapButtonEnableCombo = CreateLayoutAddBitmapButton(self, ID_BITMAPBUTTON_ENABLE_COMBO, \
                                                                        c4d.RESOURCEIMAGE_LAYERMANAGER_STATE6, idIconState2=c4d.RESOURCEIMAGE_LAYERMANAGER_STATE5, \
                                                                        tooltip='Show Dropdown', button=True, toggle=True, flags=c4d.BFH_RIGHT|c4d.BFV_CENTER, inMenu=True)
            self._bitmapButtonEnableTabs = CreateLayoutAddBitmapButton(self, ID_BITMAPBUTTON_ENABLE_TABS, \
                                                                       ID_ICON_TABS_OFF, idIconState2=ID_ICON_TABS_ON, \
                                                                       tooltip='Show Tabs', button=True, toggle=True, flags=c4d.BFH_RIGHT|c4d.BFV_CENTER, inMenu=True)
            self._bitmapButtonEnableThumbs = CreateLayoutAddBitmapButton(self, ID_BITMAPBUTTON_ENABLE_THUMBS, \
                                                                         c4d.RESOURCEIMAGE_LAYERMANAGER_STATE2, idIconState2=c4d.RESOURCEIMAGE_LAYERMANAGER_STATE1, \
                                                                         tooltip='Show Thumbnails', button=True, toggle=True, flags=c4d.BFH_RIGHT|c4d.BFV_CENTER, inMenu=True)

            self._bitmapButtonThumbSize = CreateLayoutAddBitmapButton(self, ID_BITMAPBUTTON_THUMB_SIZE, \
                                                                      bmp1=self._icons._bmpsIconThumbSize[0], \
                                                                      tooltip='Thumbnail Size', button=True, toggle=False, flags=c4d.BFH_RIGHT|c4d.BFV_CENTER, inMenu=True)

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=1):
                self.GroupBorderSpace(0, 0, 0, 3)

                self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

            self.GroupEnd()

            self._bitmapButtonAttributes = CreateLayoutAddBitmapButton(self, ID_BITMAPBUTTON_TOGGLE_ATTRIBUTES, \
                                                                       bmp1=self._icons._bmpsIconButtonAttributes[0], \
                                                                       tooltip='Toggle Cam Attributes', button=True, toggle=False, flags=c4d.BFH_RIGHT|c4d.BFV_CENTER, inMenu=True)

        self.GroupEnd()

        return True


    def InitValues(self):
        comboEnabled = ReadPrefValue(ID_PREF_COMBO_ENABLED)
        self._bitmapButtonEnableCombo.SetToggleState(comboEnabled)

        tabsEnabled = ReadPrefValue(ID_PREF_TABS_ENABLED)
        self._bitmapButtonEnableTabs.SetToggleState(tabsEnabled)

        thumbsEnabled = ReadPrefValue(ID_PREF_THUMB_ENABLED)
        self._bitmapButtonEnableThumbs.SetToggleState(thumbsEnabled)

        thumbSize = ReadPrefValue(ID_PREF_THUMB_SIZE)
        self._bitmapButtonThumbSize.SetImage(self._icons._bmpsIconThumbSize[thumbSize])
        self.Enable(ID_BITMAPBUTTON_THUMB_SIZE, thumbsEnabled)

        showAttributes = ReadPrefValue(ID_PREF_SHOW_ATTRIBUTES)
        self._bitmapButtonAttributes.SetImage(self._icons._bmpsIconButtonAttributes[showAttributes])

        return True


    def Message(self, msg, result):
        idMsg = msg.GetId()

        if idMsg == c4d.BFM_ACTION:
            if msg[c4d.BFM_ACTION_ID] == 1000 and msg[c4d.BFM_ACTION_INDRAG]:
                return True # avoid ending up in Command() during drag

        elif idMsg == 1649165891: # TODO Andreas: message?
            #print('1649165891 lost focus?')
            self.Close()

        return c4d.gui.GeDialog.Message(self, msg, result)


    def Command(self, id, msg):
        if id == ID_POPUP_MENU__MUTE_BUTTON:
            return True # mute icon button

        elif id == ID_BITMAPBUTTON_ENABLE_THUMBS:
            TogglePrefValue(ID_PREF_THUMB_ENABLED)

        elif id == ID_BITMAPBUTTON_ENABLE_COMBO:
            TogglePrefValue(ID_PREF_COMBO_ENABLED)

        elif id == ID_BITMAPBUTTON_ENABLE_TABS:
            TogglePrefValue(ID_PREF_TABS_ENABLED)

        elif id == ID_BITMAPBUTTON_THUMB_SIZE:
            StorePrefValue(ID_PREF_THUMB_SIZE, (ReadPrefValue(ID_PREF_THUMB_SIZE) + 1) % 3)

        elif id == ID_BITMAPBUTTON_TOGGLE_ATTRIBUTES:
            StorePrefValue(ID_PREF_SHOW_ATTRIBUTES, (ReadPrefValue(ID_PREF_SHOW_ATTRIBUTES) + 1) % 3)

        self.InitValues()

        self._result = id
        c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_DESIGN_POPUP_COMMAND)

        return True


    def GetResult(self):
        return self._result
