import c4d
from cams_utils_dialog import *
from cams_ids import *
from cams_ids_dialogs import *
from cams_utils_image import *
from cams_utils import *
from cams_utils_commands import *


####################################################################################################
# Class DialogPopupPerCamera #######################################################################
####################################################################################################
class PopupPerCameraIcons():

    def __init__(self):
        self._bmpIconActivate = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_STATE4, factor=1.5)

        self._bmpIconRender = LoadImage('cmd_render_cam', guiPixelRatio=2.0, factor=1.0)
        self._bmpIconRenderVP = GetIconImage(ID_ICON_COMMAND_RENDER_VP, factor=1.0)
        self._bmpIconRenderCurrentRD = GetIconImage(ID_ICON_COMMAND_RENDER_CURRENT_RD, factor=1.0)

        self._bmpIconVisDefault = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT3, factor=2.0)
        self._bmpIconVisOn = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT1, factor=2.0)
        self._bmpIconVisOff = GetIconImage(c4d.RESOURCEIMAGE_OBJECTMANAGER_DOT2, factor=2.0)

        self._bmpIconDelete = GetIconImage(ID_ICON_DELETE, factor=1.0)


class DialogPopupPerCamera(c4d.gui.GeDialog):

    _icons = None

    _doc = None

    _idxCam = None
    _objCam = None
    _nameCam = ''
    _isEditorCam = False

    _deferredRenderBusy = False

    _w = 0


    def __init__(self, icons, idxCam, objCam, isEditorCam, bmpThumb, deferredRenderBusy, initw=250):
        self._icons = icons

        self._doc = c4d.documents.GetActiveDocument() # if objCam is editor cam, objCam.GetDocument() is wrong

        self._idxCam = idxCam
        self._objCam = objCam
        if isEditorCam:
            self._nameCam = 'Editor Camera'

        else:
            self._nameCam = objCam.GetName()

        self._isEditorCam = isEditorCam

        self._bmpIcon = GetIconImage(objCam, factor=1.5)
        if bmpThumb is not None:
            self._bmpThumb = ScaleImage(bmpThumb, w=48, h=48)

        else:
            self._bmpThumb = None

        tags = GetAllCameraTags(objCam)

        bmpTagIcons = []

        for tag in tags:
            bmpTagIcons.append(GetIconImage(tag, factor=0.5))

        if len(tags) > 0:
            self._tags = zip(tags, bmpTagIcons)

        else:
            self._tags = None

        self._deferredRenderBusy = deferredRenderBusy

        self._w = initw

        c4d.gui.GeDialog.__init__(self)


    def CreateLayout(self):
        if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, initw=self._w, cols=1):
            self.GroupBorderSpace(5, 5, 5, 7)
            self.GroupBorderNoTitle(c4d.BORDER_ROUND)
            self.GroupSpace(0, 10)

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, rows=1): # title group (icon and name)
                if self._isEditorCam:
                    CreateLayoutAddBitmapButton(self, ID_POPUP_MENU__MUTE_BUTTON, bmp1=self._bmpIcon, tooltip='Editor Cam', button=False, toggle=False,
                                                flags=c4d.BFH_LEFT | c4d.BFV_CENTER, noHover=True)

                else:
                    CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_OPEN_IN_AM, bmp1=self._bmpIcon, tooltip='Select and Show in Attribute Manager', button=True, toggle=False,
                                                flags=c4d.BFH_LEFT | c4d.BFV_CENTER, noHover=True)

                self.AddStaticText(0, c4d.BFH_SCALEFIT, name='', initw=10) # spacer

                self.AddStaticText(0, c4d.BFH_CENTER | c4d.BFV_CENTER, name=self._nameCam, borderstyle=c4d.BORDER_WITH_TITLE_BOLD)

                self.AddStaticText(0, c4d.BFH_SCALEFIT, name='', initw=10) # spacer

                if self._bmpThumb is not None:
                    CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_UPDATE, bmp1=self._bmpThumb, tooltip='Update Thumbnail', button=True, toggle=False,
                                                flags=c4d.BFH_RIGHT | c4d.BFV_CENTER, noHover=True, border=c4d.BORDER_THIN_IN)

                else:
                    self.AddStaticText(0, c4d.BFH_RIGHT, name='', initw=c4d.gui.SizePix(48)) # dummy

            self.GroupEnd() # title group (icon and name)

            if self._tags is not None:
                if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_TOP, cols=2): # tag list
                    self.GroupBorderSpace(38, 0, 0, 0)

                    for idxTag, (tag, bmpIconTag) in enumerate(self._tags):
                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_OPEN_TAG_IN_AM_BASE + idxTag, bmp1=bmpIconTag, tooltip='Select and Show in Attribute Manager', button=True, toggle=False,
                                                    flags=c4d.BFH_LEFT | c4d.BFV_CENTER, noHover=True)

                        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name=tag.GetTypeName())

                self.GroupEnd() # tag list

            self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

            if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, cols=1): # Commands
                self.GroupSpace(0, 10)

                if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # First row
                    CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_ACTIVATE, bmp1=self._icons._bmpIconActivate, tooltip='Activate', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                    self.AddStaticText(0, c4d.BFH_CENTER | c4d.BFV_CENTER, name='', initw=1) # spacer to equalize smaller active icon

                    self.AddSeparatorH(0, flags=c4d.BFV_SCALEFIT)

                    CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_RENDER, bmp1=self._icons._bmpIconRender, tooltip='Render Cam', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                    self.AddSeparatorH(0, flags=c4d.BFV_SCALEFIT)

                    CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_RENDER_VP, bmp1=self._icons._bmpIconRenderVP, tooltip='Render Viewport View', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                    CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_RENDER_CURENT_RD, bmp1=self._icons._bmpIconRenderCurrentRD, tooltip='Render with Current Settings', button=True, toggle=False,
                                                flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                self.GroupEnd() # First row

                if not self._isEditorCam:
                    if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Second row
                        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_TOP, name='Editor')

                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_VIS_EDITOR_ON, bmp1=self._icons._bmpIconVisOn, tooltip='Editor Visibility: On', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_VIS_EDITOR_OFF, bmp1=self._icons._bmpIconVisOff, tooltip='Editor Visibility: Off', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_VIS_EDITOR_DEFAULT, bmp1=self._icons._bmpIconVisDefault, tooltip='Editor Visibility: Default', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                        self.AddSeparatorH(0, flags=c4d.BFV_SCALEFIT)

                        self.AddStaticText(0, c4d.BFH_LEFT | c4d.BFV_CENTER, name='Render')

                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_VIS_RENDER_ON, bmp1=self._icons._bmpIconVisOn, tooltip='Render Visibility: On', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_VIS_RENDER_OFF, bmp1=self._icons._bmpIconVisOff, tooltip='Render Visibility: Off', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)
                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_VIS_RENDER_DEFAULT, bmp1=self._icons._bmpIconVisDefault, tooltip='Render Visibility: Default', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_CENTER, noHover=True)

                    self.GroupEnd() # Second row

                    self.AddSeparatorV(0, flags=c4d.BFH_SCALEFIT)

                    if self.GroupBegin(0, c4d.BFH_CENTER | c4d.BFV_TOP, rows=1): # Third row
                        CreateLayoutAddBitmapButton(self, ID_THUMB_MENU_DELETE, bmp1=self._icons._bmpIconDelete, tooltip='Delete', button=True, toggle=False,
                                                    flags=c4d.BFH_CENTER | c4d.BFV_TOP, noHover=True)

                    self.GroupEnd() # Third row

            self.GroupEnd() # Commands

        self.GroupEnd()

        return True


    def InitValues(self):

        return True


    def CoreMessage(self, id, msg):
        if id != PLUGIN_ID_CAMS_COREMESSAGE:
            return c4d.gui.GeDialog.CoreMessage(self, id, msg)

        subIdCoreMessage = GetCoreMessageParam(msg, c4d.BFM_CORE_PAR1)

        if subIdCoreMessage == SUBID_COREMESSAGE_DEFERRED_RENDER_IDLE:
            self._deferredRenderBusy = False

        return c4d.gui.GeDialog.CoreMessage(self, id, msg)


    _dlgRequesterOpen = False

    def Message(self, msg, result):
        idMsg = msg.GetId()

        if idMsg == c4d.BFM_ACTION:
            if msg[c4d.BFM_ACTION_ID] == 1000 and msg[c4d.BFM_ACTION_INDRAG]:
                return True # avoid ending up in Command() during drag

        elif idMsg == 1649165891 and not self._dlgRequesterOpen: # TODO Andreas: message?
            #print('1649165891 lost focus?')
            self.Close()

        return c4d.gui.GeDialog.Message(self, msg, result)


    def Command(self, id, msg):
        if id == ID_POPUP_MENU__MUTE_BUTTON:
            return True # mute icon button

        elif id == ID_THUMB_MENU_ACTIVATE:
            CommandCameraActivate(self._doc, self._objCam)

        elif id == ID_THUMB_MENU_OPEN_IN_AM:
            CommandCameraOpenInAttributeManager(self._doc, self._objCam)

        elif id >= ID_THUMB_MENU_OPEN_TAG_IN_AM_BASE and id < ID_THUMB_MENU_OPEN_TAG_IN_AM_BASE + 100:
            idxTag = id - ID_THUMB_MENU_OPEN_TAG_IN_AM_BASE

            CommandCameraTagOpenInAttributeManager(self._doc, self._objCam, idxTag)

        elif id >= ID_THUMB_MENU_VIS_EDITOR_ON and id <= ID_THUMB_MENU_VIS_EDITOR_DEFAULT:
            visEditor = id - ID_THUMB_MENU_VIS_EDITOR_ON

            CommandCameraChangeVisibility(self._doc, c4d.ID_BASEOBJECT_VISIBILITY_EDITOR, self._objCam, visEditor)

            return True # do not close

        elif id >= ID_THUMB_MENU_VIS_RENDER_ON and id <= ID_THUMB_MENU_VIS_RENDER_DEFAULT:
            visRender = id - ID_THUMB_MENU_VIS_RENDER_ON

            CommandCameraChangeVisibility(self._doc, c4d.ID_BASEOBJECT_VISIBILITY_RENDER, self._objCam, visRender)

            return True # do not close

        elif id == ID_THUMB_MENU_DELETE:
            CommandCameraDelete(self._doc, self._objCam)

        elif (id >= ID_THUMB_MENU_RENDER and id <= ID_THUMB_MENU_RENDER_CURENT_RD) or \
             id == ID_THUMB_MENU_UPDATE:

            if id >= ID_THUMB_MENU_RENDER and id <= ID_THUMB_MENU_RENDER_CURENT_RD:
                self._dlgRequesterOpen = True

                ShowRenderInBackgroundInfo()

                if self._deferredRenderBusy:
                    result = ShowRenderInProgressQuestion()
                    if result == 1:
                        c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_DEFERRED_RENDER_FLUSH_QUEUE)

                    elif result == 2:
                        self._dlgRequesterOpen = False
                        return True

                self._dlgRequesterOpen = False

            subIdCoreMessage = EncodeCMPerCamSubId(id, self._idxCam)
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=subIdCoreMessage)

        self.Close()

        return True
