import c4d
from cams_utils_dialog import *
from cams_ids import *
from cams_utils_image import *
from cams_utils_pref import *


# Unique IDs registered in Plugin Cafe
# These IDs are not unique to a single plugin, but used in all plugins using these requesters.
# C4D stores dialog's size information referenc ed by these IDs.
# Thus multiple IDs are used to avoid requesters with many lines leading to
# huge requesters with only few lines in succession.
PLUGIN_IDS_REQUESTER = [ 1057818, 1057819, 1057820, 1057821, 1057822 ]


# Icon IDs
# TODO Andreas: check for actual resource symbols
DQ_ICON_QUESTION_WHITE = 300000157
DQ_ICON_QUESTION_RED = 300002170
DQ_ICON_QUESTION_ORANGE = 17190
DQ_ICON_QUESTION_DOC = 1050848
DQ_ICON_QUESTION_STRIPED = 1050586
DQ_ICON_QUESTION_CHECKER = 5616
DQ_ICON_QUESTION_FLOPPY =  100004743
DQ_ICON_RECYCLE = 440000296
DQ_ICON_EXCLAMATION_WHITE_BG = 1031728
DQ_ICON_EXCLAMATION_ORANGE_BAM = 1018880
DQ_ICON_LIGHTBULB = 5102
DQ_ICON_TRASHCAN = 13685
DQ_ICON_GLOBE_ORANGE = 450000017
DQ_ICON_RADIO_ACTIVE_ORANGE = 13663
DQ_ICON_X_GREY_BG = 13957

DQ_ICON_ERROR = DQ_ICON_EXCLAMATION_WHITE_BG # 'dialogIcon_Delete'


def GetRequesterChoice(id):
    if type(id) == str:
        id = MyHash(id)

    bcWorldPrefs = c4d.plugins.GetWorldPluginData(PLUGIN_ID_CAMS)

    bcRequesters = bcWorldPrefs.GetContainerInstance(ID_PREF_REQUESTER_CHOICES)

    if bcRequesters is None:
        return None

    choice = None

    if bcRequesters.FindIndex(id)['index'] != -1:
        choice = bcRequesters.GetInt32(id)

    return choice

def SetRequesterChoice(id, choice):
    bcWorldPrefs = c4d.plugins.GetWorldPluginData(PLUGIN_ID_CAMS)

    bcRequesters = bcWorldPrefs.GetContainerInstance(ID_PREF_REQUESTER_CHOICES)
    if bcRequesters is None:
        bcWorldPrefs.SetContainer(ID_PREF_REQUESTER_CHOICES, c4d.BaseContainer())

        bcRequesters = bcWorldPrefs.GetContainerInstance(ID_PREF_REQUESTER_CHOICES)

    bcRequesters.SetInt32(id, choice)

    c4d.plugins.SetWorldPluginData(PLUGIN_ID_CAMS, bcWorldPrefs, add=True) # actually redundant


def OpenRequester(title='', msgs=[''], buttonLabels=['Ok'], icon=DQ_ICON_QUESTION_WHITE,
                  twoButtonRows=True, lastButtonExtra=True, idxFocus=-1, idxAbort=-1, scaleIcon=2.0,
                  wIcon=c4d.gui.SizePix(65), wButton=c4d.gui.SizePix(65), hButton=c4d.gui.SizePix(20),
                  id=None, saveDefault=False,
                  xpos=-2, ypos=-2):
    dlg = DialogQuestion(title, msgs, buttonLabels, icon, twoButtonRows, lastButtonExtra, idxFocus, idxAbort, scaleIcon, wIcon, wButton, hButton, id, saveDefault)

    choice = GetRequesterChoice(dlg.GetRequesterId())
    if choice is not None:
        return choice

    idxSize = max(3, len(msgs)) - 3
    if twoButtonRows:
        idxSize += 1

    idxSize = min(idxSize, len(PLUGIN_IDS_REQUESTER) - 1)

    idPluginRequester = PLUGIN_IDS_REQUESTER[idxSize]

    dlg.Open(c4d.DLG_TYPE_MODAL, idPluginRequester, xpos=xpos, ypos=ypos, defaultw=0, defaulth=0)

    return dlg.GetResult()


def OpenQuestionDialog(title='', msgs=[''], buttonLabels=['Ok'], icon=DQ_ICON_QUESTION_WHITE,
                       twoButtonRows=False, lastButtonExtra=True, idxFocus=-1, idxAbort=-1, scaleIcon=2.0,
                       wIcon=c4d.gui.SizePix(65), wButton=c4d.gui.SizePix(65), hButton=c4d.gui.SizePix(20),
                       id=None, saveDefault=False,
                       xpos=-2, ypos=-2):
    return OpenRequester(title, msgs, buttonLabels, icon, twoButtonRows, lastButtonExtra, idxFocus, idxAbort, scaleIcon, wIcon, wButton, hButton, id, saveDefault, xpos, ypos)


DQ_RESULT_YES = 0
DQ_RESULT_NO = 1
def OpenYesNoDialog(title='', msgs=[''], buttonLabels=['Yes', 'No'], icon=DQ_ICON_QUESTION_WHITE,
                    twoButtonRows=False, lastButtonExtra=False, idxFocus=1, idxAbort=1, scaleIcon=2.0,
                    wIcon=c4d.gui.SizePix(65), wButton=c4d.gui.SizePix(65), hButton=c4d.gui.SizePix(20),
                    id=None, saveDefault=False,
                    xpos=-2, ypos=-2):
    return OpenRequester(title, msgs, buttonLabels, icon, twoButtonRows, lastButtonExtra, idxFocus, idxAbort, scaleIcon, wIcon, wButton, hButton, id, saveDefault, xpos, ypos)


def OpenErrorDialog(title='ERROR', msgs=[''], buttonLabels=['Ok'], icon=DQ_ICON_ERROR,
                    twoButtonRows=False, lastButtonExtra=False, idxFocus=0, idxAbort=0, scaleIcon=2.0,
                    wIcon=c4d.gui.SizePix(65), wButton=c4d.gui.SizePix(65), hButton=c4d.gui.SizePix(20),
                    id=None, saveDefault=False,
                    xpos=-2, ypos=-2):
    return OpenRequester(title, msgs, buttonLabels, icon, twoButtonRows, lastButtonExtra, idxFocus, idxAbort, scaleIcon, wIcon, wButton, hButton, id, saveDefault, xpos, ypos)


def OpenInfoDialog(title='', msgs=[''], buttonLabels=['Ok'], icon=DQ_ICON_LIGHTBULB,
                   twoButtonRows=False, lastButtonExtra=False, idxFocus=0, idxAbort=0, scaleIcon=2.0,
                   wIcon=c4d.gui.SizePix(65), wButton=c4d.gui.SizePix(65), hButton=c4d.gui.SizePix(20),
                   id=None, saveDefault=False,
                   xpos=-2, ypos=-2):
    return OpenRequester(title, msgs, buttonLabels, icon, twoButtonRows, lastButtonExtra, idxFocus, idxAbort, scaleIcon, wIcon, wButton, hButton, id, saveDefault, xpos, ypos)


class DialogQuestion(c4d.gui.GeDialog):

    ID_BASE_BUTTON = 1000
    ID_CHECKBOX_SAVEDEFAULT = 2000

    _id = None
    _title = ''
    _msgs = ['']
    _buttonLabels = []
    _icon = None
    _twoButtonRows = True
    _lastButtonExtra = True
    _idxFocus = -1
    _idxAbort = -1
    _wIcon = 32
    _wButton = 0
    _hButton = 0
    _saveDefault = False

    _result = -1


    def __init__(self, title='', msgs=[''], buttonLabels=['Ok'], icon=DQ_ICON_QUESTION_WHITE,
                 twoButtonRows=True, lastButtonExtra=True, idxFocus=-1, idxAbort=-1,
                 scaleIcon=1.0, wIcon=c4d.gui.SizePix(65),
                 wButton=c4d.gui.SizePix(65), hButton=c4d.gui.SizePix(20),
                 id=None, saveDefault=False):

        if id is None:
            self._id = MyHash(msgs[0])

        elif type(id) == str:
            self._id = MyHash(id)

        else:
            self._id = id

        self._title = title
        self._msgs = msgs
        self._buttonLabels = buttonLabels

        if icon is None:
            self._icon = None

        elif type(icon) == c4d.bitmaps.BaseBitmap:
            self._icon = icon
            scaleIcon = 1.0

        elif type(icon) == int:
            self._icon = GetIconImage(icon)

        elif type(icon) == str:
            self._icon = LoadImage(icon)
            scaleIcon = 1.0

        else:
            self._icon = GetIconImage(DQ_ICON_QUESTION_RED)

        if self._icon is not None and scaleIcon != 1.0:
            self._icon = ScaleImage(self._icon, factor=scaleIcon)

        self._twoButtonRows = twoButtonRows
        self._lastButtonExtra = lastButtonExtra
        self._idxFocus = idxFocus
        self._idxAbort = idxAbort
        self._wIcon = wIcon
        self._wButton = wButton
        self._hButton = hButton
        self._saveDefault = saveDefault

        self._result = -1

        c4d.gui.GeDialog.__init__(self)


    def CreateLayout(self):
        numMainCols = 1
        if self._icon is not None:
            numMainCols = 2

        numButtonCols = 1
        buttonLabels = self._buttonLabels
        flagsMainButtons = c4d.BFH_SCALE | c4d.BFH_CENTER | c4d.BFV_BOTTOM
        if self._lastButtonExtra:
            numButtonCols = 2
            buttonLabels = self._buttonLabels[:-1]
            flagsMainButtons = c4d.BFH_SCALEFIT | c4d.BFV_BOTTOM

        numCols = len(buttonLabels)
        if self._twoButtonRows:
            numCols = len(buttonLabels) // 2

        if len(self._title) > 0:
            self.SetTitle('{0}: {1}'.format(PLUGIN_NAME, self._title))

        else:
            self.SetTitle('{0}'.format(PLUGIN_NAME))

        if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, cols=1):
            self.GroupBorderSpace(10, 12, 10, 3)

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT, cols=numMainCols):
                self.GroupSpace(15, 0)

                if self._icon is not None:
                    CreateLayoutAddBitmapButton(self, 0, bmp1=self._icon, noHover=True, w=self._wIcon)

                if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_SCALE | c4d.BFV_CENTER, cols=1):
                    for msg in self._msgs:
                        if type(msg) == str:
                            self.AddStaticText(0, c4d.BFH_SCALEFIT, name=msg)

                        elif type(msg) == tuple:
                            self.AddStaticText(0, c4d.BFH_SCALEFIT, name=msg[0], borderstyle=msg[1])

                        else:
                            PrintError('Unknown message line type! {1}'.format(type(msg)))

                self.GroupEnd()

            self.GroupEnd()

            self.AddStaticText(0, c4d.BFH_SCALEFIT, name='') # spacer

            if self._saveDefault:
                self.AddCheckbox(self.ID_CHECKBOX_SAVEDEFAULT, c4d.BFH_LEFT, initw=0, inith=0, name='Do not show again')

            if self.GroupBegin(0, c4d.BFH_SCALEFIT | c4d.BFV_BOTTOM, cols=numButtonCols):

                if self.GroupBegin(0, flagsMainButtons, cols=numCols):
                    for idxLabel, label in enumerate(buttonLabels):
                        self.AddButton(self.ID_BASE_BUTTON + idxLabel, c4d.BFH_CENTER, initw=self._wButton, inith=self._hButton, name=label)

                self.GroupEnd()

                if self._lastButtonExtra:
                    self.AddButton(self.ID_BASE_BUTTON + len(self._buttonLabels) - 1, c4d.BFH_CENTER | c4d.BFV_SCALEFIT, initw=self._wButton, name=self._buttonLabels[-1])

            self.GroupEnd()

        self.GroupEnd()

        if self._idxFocus != -1:
            self.Activate(self._idxFocus)

        return True


    def InitValues(self):
        if self._saveDefault:
            self.SetBool(self.ID_CHECKBOX_SAVEDEFAULT, False)

        return True


    def AskClose(self):
        if self._result == -1:
            if self._idxAbort == -1:
                return True

            else:
                self._result = self._idxAbort

        if self._saveDefault and self.GetBool(self.ID_CHECKBOX_SAVEDEFAULT):
            SetRequesterChoice(self._id, self._result)

        return False


    def Command(self, id, msg):
        if id >= self.ID_BASE_BUTTON and id < self.ID_BASE_BUTTON + len(self._buttonLabels):
            self._result = id - self.ID_BASE_BUTTON

            self.Close()

        return True


    def GetRequesterId(self):
        return self._id


    def GetResult(self):
        return self._result
