import math
import c4d
from cams_ids import *
from cams_ids_dialogs import *
from cams_utils_image import *
from cams_utils_pref import *
from cams_utils import *
from cams_utils_commands import *
from cams_deferred_render import DeferredRendering


def MsgDataDestroyGlobals():
    global g_messageData

    if g_messageData is None:
        return

    g_messageData.KillDeferredRenderer()

    g_messageData = None


g_messageData = None


def RegisterMessageData():
    res =  c4d.plugins.RegisterMessagePlugin(PLUGIN_ID_CAMS_MESSAGEDATA, str=PLUGIN_NAME, info=0, dat=MessageDataCams())
    if not res:
        PrintError('Failed to register MessageData {0}'.format(PLUGIN_ID_CAMS_MESSAGEDATA))
        return False

    return True


####################################################################################################
# Class MessageDataCams ############################################################################
####################################################################################################
class MessageDataCams(c4d.plugins.MessageData):
    _deferredRenderer = None

    _bmpsProofSheet = {}

    _docLast = None


    def __init__(self):
        global g_messageData

        g_messageData = self


    def ProofSheetDrawOverlay(self, renderJobResult):
        if not ReadPrefValue(ID_PREF_PROOF_SHEET_OVERLAY_NAME):
            return

        bcDoc = GetDocBC()

        dictCameras = CameraBCToDict(bcDoc)

        objCam = dictCameras[renderJobResult._idJob][0]
        isEditorCam = dictCameras[renderJobResult._idJob][3]

        if isEditorCam:
            name = 'Editor Cam'

        else:
            name = objCam.GetName()

        renderJobResult._bmpResult = DrawTextOverlay(renderJobResult._bmpResult, sLabel=name, sizeFactor=2.0, bold=False, italic=False)


    def CheckProofSheetComplete(self):
        for bmp in self._bmpsProofSheet.values():
            if bmp is None:
                return False

        return True


    def MergeProofSheet(self):
        wSingle, hSingle = list(self._bmpsProofSheet.values())[0].GetSize()

        numImages = len(self._bmpsProofSheet)

        cols = math.ceil(math.sqrt(numImages))

        rows = math.ceil(numImages / cols)

        bmpProofSheet = c4d.bitmaps.BaseBitmap()

        wProofSheet, hProofSheet = cols * wSingle, rows * hSingle

        bmpProofSheet.Init(wProofSheet, hProofSheet)

        idxCol = 0
        idxRow = 0
        # TODO support alpha
        for bmpSingle in self._bmpsProofSheet.values():
            CopyImageIntoImage(bmpSingle, bmpProofSheet, idxCol * wSingle, idxRow * hSingle)

            idxCol += 1

            if idxCol >= cols:
                idxCol = 0
                idxRow += 1

        return bmpProofSheet


    def SaveProofSheet(self, renderJobResult, bmpProofSheet):
        docOrig = renderJobResult._docOrig

        idxFrame = docOrig.GetTime().GetFrame(docOrig.GetFps())
        rd = renderJobResult._docRender.GetActiveRenderData()

        rpd = {'_doc': docOrig, '_rData': rd, '_rBc': rd.GetData(), '_frame': idxFrame}
        filename = c4d.modules.tokensystem.FilenameConvertTokensFilter(rd[c4d.RDATA_PATH], rpd, [])

        savebits = BMP_SAVEBIT_NONE
        if rd[c4d.RDATA_ALPHACHANNEL]:
            savebits |= c4d.SAVEBIT_ALPHA

        formatImg = rd[c4d.RDATA_FORMAT]

        if formatImg == c4d.FILTER_TIF and filename[-4:] != '.tif':
            filename += '.tif'

        elif formatImg == c4d.FILTER_JPG and filename[-4:] != '.jpg':
            filename += '.jpg'

        elif formatImg == c4d.FILTER_PNG and filename[-4:] != '.png':
            filename += '.png'

        elif formatImg == c4d.FILTER_BMP and filename[-4:] != '.bmp':
            filename += '.bmp'

        path, _ = os.path.split(filename)

        if not os.path.exists(path):
            os.makedirs(path)

        result = bmpProofSheet.Save(filename, format=formatImg, data=None, savebits=savebits)

        if result != c4d.IMAGERESULT_OK:
            PrintError('Failed to save proof sheet: {0}'.format(filename))


    def KillDeferredRenderer(self):
        if self._deferredRenderer is not None:
            self._deferredRenderer.StopRenderThread()

        self._deferredRenderer = None


    def QueueRenderJobImage(self, doc, guid, objCam, isEditorCam, forProofSheet=False, path='', useCurrentRD=False, useViewportRender=False):
        docClone, objCamClone = CloneDocumentFocussed(doc, objCam, cloneOnly=isEditorCam)
        if docClone is None:
            PrintError('Failed to clone document')
            return False

        flags = 0

        if forProofSheet:
            target = ReadPrefValue(ID_PREF_PROOF_SHEET_TARGET)
            sizeFactor = ReadPrefValue(ID_PREF_PROOF_SHEET_SIZE_FACTOR)
            flags |= 0x40000000 # render image for proof sheet

        else:
            target = ReadPrefValue(ID_PREF_RENDER_ALL_TARGET)
            sizeFactor = 1
            flags |= 0x80000000 # render image

        bdRenderClone = docClone.GetRenderBaseDraw()
        bdRenderClone.SetSceneCamera(objCamClone)

        if isEditorCam:
            objCamClone = docClone.GetRenderBaseDraw().GetEditorCamera()
            objCamClone.SetName('Editor Cam')

        if useCurrentRD:
            rdRender = docClone.GetActiveRenderData()

        elif useViewportRender:
            rdCurrent = docClone.GetActiveRenderData()

            rdRender = CreateDefaultRenderDataViewportRender(w=rdCurrent[c4d.RDATA_XRES], h=rdCurrent[c4d.RDATA_YRES], docToInsert=docClone)

            docClone.SetActiveRenderData(rdRender)

        else:
            rdRender = GetImageRenderData(docClone, activate=True, forProofSheet=forProofSheet)

        if DetectProgressiveRender(docClone):
            ShowProgressiveRenderError()
            return False

        rdRender[c4d.RDATA_PATH] = path

        if sizeFactor != 1:
            rdRender[c4d.RDATA_XRES] = int(rdRender[c4d.RDATA_XRES] / sizeFactor)
            rdRender[c4d.RDATA_YRES] = int(rdRender[c4d.RDATA_YRES] / sizeFactor)

        if target == 0 or target == 2:
            flags |= 0x01000000 # open in Picture Viewer
            rdRender[c4d.RDATA_SAVEIMAGE] = False

        if target == 1 or target == 2:
            flags |= 0x02000000 # save image
            rdRender[c4d.RDATA_SAVEIMAGE] = True

        if self._deferredRenderer is not None:
            self._deferredRenderer.AddJob(doc, guid, docClone, flags=flags, convertMultipassBitmap=False)

        else:
            PrintError('WTF? No deferred renderer?!?!?!')

        return True


    def ReceiveRenderResultRenderedImage(self, renderJobResult):
        # Saving of image has already been done during RenderDocument and save flag in RenderData
        if renderJobResult._flags & 0x01000000 == 0x01000000:
            c4d.bitmaps.ShowBitmap(renderJobResult._bmpResult)


    def ReceiveRenderResultProofSheet(self, renderJobResult):
        if renderJobResult._idJob not in self._bmpsProofSheet:
            return

        self.ProofSheetDrawOverlay(renderJobResult)

        self._bmpsProofSheet[renderJobResult._idJob] = renderJobResult._bmpResult

        if not self.CheckProofSheetComplete():
            return

        bmpProofSheet = self.MergeProofSheet()

        self._bmpsProofSheet = {}

        if renderJobResult._flags & 0x01000000 == 0x01000000:
            c4d.bitmaps.ShowBitmap(bmpProofSheet)

        if renderJobResult._flags & 0x02000000 == 0x02000000:
            self.SaveProofSheet(renderJobResult, bmpProofSheet)


    def CommandRenderAllCameras(self, asProofSheet=False, useViewportRender=False, useCurrentRD=False):
        doc = c4d.documents.GetActiveDocument()

        path = doc.GetDocumentPath()
        if path is None or len(path) <= 2:
            if asProofSheet:
                title = 'Choose directory to save proof sheet into...'

            else:
                title = 'Choose directory to save rendered images into...'

            path = c4d.storage.LoadDialog(c4d.FILESELECTTYPE_ANYTHING, title, c4d.FILESELECT_DIRECTORY, def_path=path)

            if path is None  or len(path) <= 2:
                return

        if asProofSheet:
            filename = ReadPrefValue(ID_PREF_PROOF_SHEET_FILENAME)
            includeEditorCam = ReadPrefValue(ID_PREF_PROOF_SHEET_EDITOR_CAM)

        else:
            filename = ReadPrefValue(ID_PREF_RENDER_ALL_FILENAME)
            includeEditorCam = ReadPrefValue(ID_PREF_RENDER_ALL_EDITOR_CAM)

        if '$datetime' in filename:
            tNow = time.localtime()

            filename = filename.replace('$datetime', '{0}{1:02}{2:02}_{3:02}{4:02}{5:02}'.format(tNow.tm_year, tNow.tm_mon, tNow.tm_mday, tNow.tm_hour, tNow.tm_min, tNow.tm_sec))

        path = os.path.join(path, filename)

        bcDoc = GetDocBC(doc)

        cameras = CameraBCToList(bcDoc)

        if asProofSheet:
            self._bmpsProofSheet = {}

        for objCam, _, _, guid, isEditorCam in cameras:
            if isEditorCam and not includeEditorCam:
                continue

            if not self.QueueRenderJobImage(doc, guid, objCam, isEditorCam, forProofSheet=asProofSheet, path=path, useViewportRender=useViewportRender, useCurrentRD=useCurrentRD):
                break

            if asProofSheet:
                self._bmpsProofSheet[guid] = None


    def CommandRenderCamera(self, doc, cameras, idxCam, objCam, isEditorCam, useCurrentRD=False, useViewportRender=False):
        guid = cameras[idxCam][3]

        path = doc.GetDocumentPath()
        if path is None or len(path) <= 2:
            title = 'Choose directory to save rendered image into...'

            path = c4d.storage.LoadDialog(c4d.FILESELECTTYPE_ANYTHING, title, c4d.FILESELECT_DIRECTORY, def_path=path)

            if path is None  or len(path) <= 2:
                return

        filename = ReadPrefValue(ID_PREF_RENDER_ALL_FILENAME)

        if '$datetime' in filename:
            tNow = time.localtime()

            filename = filename.replace('$datetime', '{0}{1:02}{2:02}_{3:02}{4:02}{5:02}'.format(tNow.tm_year, tNow.tm_mon, tNow.tm_mday, tNow.tm_hour, tNow.tm_min, tNow.tm_sec))

        path = os.path.join(path, filename)

        self.QueueRenderJobImage(doc, guid, objCam, isEditorCam, forProofSheet=False, path=path, useCurrentRD=useCurrentRD, useViewportRender=useViewportRender)


    def CommandFromCommandPanel(self, idCommand):
        if idCommand == 0:
            return

        elif idCommand >= ID_EDITOR_VISIBILITY_ALL_DEFAULT and idCommand <= ID_RENDER_VISIBILITY_ALL_OFF:
            CommandVisibilityAll(idCommand)

        elif idCommand == ID_RENDER_ALL_TO_PV:
            self.CommandRenderAllCameras()

        elif idCommand == ID_RENDER_ALL_TO_PV_VP:
            self.CommandRenderAllCameras(useViewportRender=True)

        elif idCommand == ID_RENDER_ALL_TO_PV_CURRENT_RD:
            self.CommandRenderAllCameras(useCurrentRD=True)

        elif idCommand == ID_RENDER_PROOF_SHEET:
            self.CommandRenderAllCameras(asProofSheet=True)

        elif idCommand == ID_RENDER_PROOF_SHEET_VP:
            self.CommandRenderAllCameras(asProofSheet=True, useViewportRender=True)

        elif idCommand == ID_RENDER_PROOF_SHEET_CURRENT_RD:
            self.CommandRenderAllCameras(asProofSheet=True, useCurrentRD=True)

        elif idCommand >= ID_CREATE_CAMERA_STANDARD and idCommand <= ID_CREATE_CAMERA_STANDARD + 999:
            CommandCreateCamera(idCommand)


    def CommandFromPerCameraPopup(self, subIdCoreMessage):
        idCommand, idxCam = DecodeCMPerCamSubId(subIdCoreMessage)

        doc = c4d.documents.GetActiveDocument()

        bcDoc = GetDocBC(doc)

        cameras = CameraBCToList(bcDoc)

        if idxCam >= len(cameras):
            PrintError('idxCam out of range')
            return

        objCam = cameras[idxCam][0]
        isEditorCam = cameras[idxCam][4]

        if idCommand == ID_THUMB_MENU_RENDER:
            self.CommandRenderCamera(doc, cameras, idxCam, objCam, isEditorCam)

        elif idCommand == ID_THUMB_MENU_RENDER_VP:
            self.CommandRenderCamera(doc, cameras, idxCam, objCam, isEditorCam, useViewportRender=True)

        elif idCommand == ID_THUMB_MENU_RENDER_CURENT_RD:
            self.CommandRenderCamera(doc, cameras, idxCam, objCam, isEditorCam, useCurrentRD=True)

        # elif other commands in DialogMain


    def CoreMessageReceiveRenderResult(self):
        renderJobResult = self._deferredRenderer.GetResult()

        while renderJobResult is not None:
            if renderJobResult.HasError():
                renderJobResult.PrintError()

                self._deferredRenderer.Reset()

                renderJobResult = self._deferredRenderer.GetResult()
                continue

            if renderJobResult._flags & 0xFFFF0000 == 0: # update thumb
                PrintError('Received thumb update render result in MessageData')

            elif renderJobResult._flags & 0x80000000 == 0x80000000:
                self.ReceiveRenderResultRenderedImage(renderJobResult)

            elif renderJobResult._flags & 0x40000000 == 0x40000000: # rendering proof sheet
                self.ReceiveRenderResultProofSheet(renderJobResult)

            renderJobResult = self._deferredRenderer.GetResult()


    def CoreMessageDeferredRenderFlushQueue(self):
        self._deferredRenderer.Reset()


    def CoreMessageCams(self, msg):
        if self._deferredRenderer is None:
            self._deferredRenderer = DeferredRendering(PLUGIN_ID_CAMS_COREMESSAGE,
                                                       SUBID_COREMESSAGE_RENDER_FINISHED_MSGDATA, SUBID_COREMESSAGE_CAMERA_LIST_CHANGE,
                                                       SUBID_COREMESSAGE_DEFERRED_RENDER_IDLE, SUBID_COREMESSAGE_DEFERRED_RENDER_BUSY)

        subIdCoreMessage = GetCoreMessageParam(msg, c4d.BFM_CORE_PAR1)

        if subIdCoreMessage >= SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE and subIdCoreMessage < SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE + 1000000:
            self.CommandFromCommandPanel(subIdCoreMessage - SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE)

        elif subIdCoreMessage >= SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE_KEEP and subIdCoreMessage < SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE_KEEP + 1000000:
            self.CommandFromCommandPanel(subIdCoreMessage - SUBID_COREMESSAGE_DLG_COMMAND_PANEL_BASE_KEEP)

        elif subIdCoreMessage >= SUBID_COREMESSAGE_DLG_PER_CAM_BASE and subIdCoreMessage < SUBID_COREMESSAGE_DLG_PER_CAM_BASE + 1000000:
            self.CommandFromPerCameraPopup(subIdCoreMessage)

        elif subIdCoreMessage == SUBID_COREMESSAGE_RENDER_FINISHED_MSGDATA:
            self.CoreMessageReceiveRenderResult()

        elif subIdCoreMessage == SUBID_COREMESSAGE_DEFERRED_RENDER_FLUSH_QUEUE:
            self.CoreMessageDeferredRenderFlushQueue()


    def CheckActiveCamera(self, doc):
        bcDoc = GetDocBC(doc)

        objCamActiveLast = bcDoc.GetLink(ID_CAM_LAST_ACTIVE)
        objCamActive, isEditorCam = GetActiveCamera(doc=None)

        if objCamActiveLast == objCamActive:
            return

        bcDoc.SetLink(ID_CAM_LAST_ACTIVE, objCamActive)

        cameras = CameraBCToList(bcDoc)

        idxCamToUpdate = GetCameraIndex(cameras, objCamActiveLast)

        if idxCamToUpdate is not None:
            bcDoc.SetInt32(ID_CAM_IDX_UPDATE_THUMB_LAST, idxCamToUpdate)

        c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_CAMERA_ACTIVE_CHANGE)


    def UpdateCameraData(self, doc):
        bcDoc = GetDocBC(doc)

        flagsHDirtyLast = bcDoc.GetInt32(ID_DOC_LAST_HDIRTY)
        flagsHDirty = doc.GetHDirty(HDIRTYFLAGS_OBJECT | HDIRTYFLAGS_OBJECT_HIERARCHY)

        docChanged = self._docLast is None or not self._docLast.IsAlive() or doc != self._docLast

        if flagsHDirty == flagsHDirtyLast and not docChanged:
            return False

        cameras = GetObjectListByType(doc=doc, typeObj=c4d.Ocamera)

        objCamEditor = GetEditorCamera(doc)

        cameraNewOrChanged = False

        for objCam in cameras:
            cameraNewOrChanged |= StoreCamera(bcDoc, objCam)

        cameraNewOrChanged |= StoreCamera(bcDoc, objCamEditor, isEditorCam=True)

        cameras.append(objCamEditor) # prevent editor cam being garbage collected

        cameraLost = GarbageCollectCameraBC(bcDoc, cameras)

        bcDoc.SetInt32(ID_DOC_LAST_HDIRTY, flagsHDirty)

        if cameraNewOrChanged or cameraLost or docChanged:
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_CAMERA_LIST_CHANGE)

        self._docLast = doc

        return True


    def CoreMessageEvmsgChange(self):
        doc = c4d.documents.GetActiveDocument()

        dataChange = self.UpdateCameraData(doc)

        if not dataChange:
            self.CheckActiveCamera(doc)


    def CoreMessage(self, id, bc):
        if not ReadPrefValue(ID_PREF_PLUGIN_ENABLED):
            return True

        if id == c4d.EVMSG_CHANGE:
            self.CoreMessageEvmsgChange()

        elif id == PLUGIN_ID_CAMS_COREMESSAGE:
            self.CoreMessageCams(bc)

        return True
