import c4d
from cams_utils_c4d_version import *
from cams_ids import *
from cams_utils_pref import *
from cams_utils import *


ID_GROUP_PREF_DESIGN                   = 2501
ID_GROUP_PREF_THUMB                    = 2502
ID_GROUP_PREF_ACTIVE_CAMERA            = 2503
ID_GROUP_PREF_THUMB_OVERLAY            = 2504
ID_GROUP_PREF_DESIGN_CAMERA            = 2505
ID_GROUP_PREF_DESIGN_EXTRA             = 2506
ID_GROUP_PREF_OVERLAY_COLORS           = 2507
ID_GROUP_PREF_CREATE_CAMERAS           = 2508
ID_GROUP_PREF_PROOF_SHEET              = 2509
ID_GROUP_PREF_RENDER_ALL               = 2510
ID_GROUP_PREF_PROOF_SHEET_FILENAME     = 2511
ID_GROUP_PREF_RENDER_ALL_FILENAME      = 2512
ID_GROUP_PREF_IN_PALETTE               = 2513


def RegisterPrefs(res):
    global __res__

    __res__ = res

    if c4d.GetC4DVersion() // 1000 >= 24:
        idParent = 1056498 # TODO Andreas: Look for "Extensions" group # c4d.PREFS_EXTENSIONS = 1056498 not working so far

    else:
        idParent = 0

    res = c4d.plugins.RegisterPreferencePlugin(PLUGIN_ID_CAMS_PREFS,
                                               g=PreferenceDataCams, name=PLUGIN_NAME, description='camspreferences',
                                               parentid=idParent, sortid=0)
    if not res:
        PrintError('Failed to register PreferenceData')
        return False

    # Workaround for above R24 issue. Thanks to Maxime from Maxon!!!
    plugin = c4d.plugins.FindPlugin(PLUGIN_ID_CAMS_PREFS, c4d.PLUGINTYPE_PREFS)
    if plugin is not None:
        plugin.GetDataInstance().SetInt32(700, 0)
        plugin.GetDataInstance().SetInt32(701, idParent)

    return True


####################################################################################################
# Class PreferenceDataCams #########################################################################
####################################################################################################
class PreferenceDataCams(c4d.plugins.PreferenceData):

    def InitPrefValue(self, description, id, dtype, bcWorldPrefs, default):
        self.InitPreferenceValue(id, default, description, c4d.DescID(c4d.DescLevel(id, dtype, 0)), bcWorldPrefs)


    def Init(self, node, description=None):
        bcWorldPrefs = GetWorldPrefs()

        for idPV, (dataType, valDefault, _, Set, _) in TABLE_PREFS.items():
            self.InitPrefValue(description, idPV, dataType, bcWorldPrefs, valDefault)

        return True


    def GetDDescriptionCreateCheckbox(self, node, description, descId, name, parentId, valDefault=True):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetString(c4d.DESC_SHORT_NAME, name)
        bc.SetInt32(c4d.DESC_DEFAULT, valDefault)
        bc.SetInt32(c4d.DESC_ANIMATE, False)
        bc.SetBool(c4d.DESC_REMOVEABLE, False)

        result = description.SetParameter(descId, bc, parentId)
        if not result:
            print('{0} ERROR: FAILED TO CREATE CHECKBOX: {1}'.format(PLUGIN_NAME, name))

        return result


    def GetDDescriptionCreateCombo(self, node, description, descId, name, parentId, bcContent, valDefault=0):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetString(c4d.DESC_SHORT_NAME, name)
        bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_CYCLE)
        bc.SetInt32(c4d.DESC_DEFAULT, valDefault)
        bc.SetInt32(c4d.DESC_ANIMATE, False)
        bc.SetBool(c4d.DESC_REMOVEABLE, False)
        bc.SetContainer(c4d.DESC_CYCLE, bcContent)

        result = description.SetParameter(descId, bc, parentId)
        if not result:
            print('{0} ERROR: FAILED TO CREATE COMBO: {1}'.format(PLUGIN_NAME, name))

        return result


    def GetDDescriptionCreateGroup(self, description, groupDescId, name, parentId, numColumns=1, defaultOpen=True):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetInt32(c4d.DESC_COLUMNS, numColumns)
        bc.SetBool(c4d.DESC_SCALEH, True)

        if defaultOpen:
            bc.SetInt32(c4d.DESC_DEFAULT, -1)

        return description.SetParameter(groupDescId, bc, c4d.DescID(c4d.DescLevel(parentId)))


    def GetDDescriptionCreateLong(self, description, descId, name, parentId, unit=c4d.DESC_UNIT_INT, valDefault=50, valMin=0, valMax=100, step=1, slider=True, sliderMin=0, sliderMax=100):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetString(c4d.DESC_SHORT_NAME, name)

        if slider:
            bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_LONGSLIDER)
            bc.SetInt32(c4d.DESC_MINSLIDER, sliderMin)
            bc.SetInt32(c4d.DESC_MAXSLIDER, sliderMax)

        else:
            bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_LONG)

        bc.SetInt32(c4d.DESC_DEFAULT, valDefault)
        bc.SetInt32(c4d.DESC_MIN, valMin)
        bc.SetInt32(c4d.DESC_MAX, valMax)
        bc.SetInt32(c4d.DESC_STEP, step)
        bc.SetInt32(c4d.DESC_UNIT, unit)
        bc.SetInt32(c4d.DESC_ANIMATE, False)
        bc.SetBool(c4d.DESC_REMOVEABLE, False)

        return description.SetParameter(descId, bc, parentId)


    def GetDDescriptionCreateButton(self, description, descId, name, parentId):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BUTTON)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_BUTTON)

        return description.SetParameter(descId, bc, c4d.DescID(c4d.DescLevel(parentId)))


    def GetDDescriptionCreateColor(self, description, descId, name, parentId, valDefault=c4d.Vector(1.0, 0.0, 1.0)):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_COLOR)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetVector(c4d.DESC_DEFAULT, valDefault)
        bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_COLOR)

        return description.SetParameter(descId, bc, c4d.DescID(c4d.DescLevel(parentId)))


    def GetDDescriptionCreateString(self, description, descId, name, parentId, valDefault='', static=False):
        bc = c4d.GetCustomDataTypeDefault(c4d.DTYPE_STRING)

        bc.SetString(c4d.DESC_NAME, name)
        bc.SetString(c4d.DESC_DEFAULT, valDefault)
        if static:
            bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_STATICTEXT)

        else:
            bc.SetInt32(c4d.DESC_CUSTOMGUI, c4d.CUSTOMGUI_STRING)

        bc.SetBool(c4d.DESC_SCALEH, True)

        return description.SetParameter(descId, bc, c4d.DescID(c4d.DescLevel(parentId)))


    def GetDDescriptionCreateComboThumbSize(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(0, 'Small')
        bc.SetString(1, 'Medium')
        bc.SetString(2, 'Large')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionCreateComboThumbAspectRatio(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(0, 'Square')
        bc.SetString(1, 'Viewport')
        bc.SetString(2, 'Viewport Safe Frame')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionCreateComboThumbInterval(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(0, 'No Automatic Updates')
        bc.SetString(1, 'Camera Switch')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionCreateComboThumbMarker(self, node, description, descId, name, idParent, valDefault):
        availableMarker = GetAvailableMarker()

        bc = c4d.BaseContainer()

        for marker in availableMarker.values():
            bc.SetString(marker[2], marker[0])

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionCreateCreateCameras(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(0, 'None')

        if HaveRedshift():
            bc.SetString(1, 'Redshift')

        if HaveCorona():
            bc.SetString(2, 'Corona')

        if HaveOctane():
            bc.SetString(3, 'Octane')

        if HaveVray():
            bc.SetString(4, 'Vray')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionProofSheetSize(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(1, '1/1 (Original Size)')
        bc.SetString(2, '1/2')
        bc.SetString(3, '1/3')
        bc.SetString(4, '1/4')
        bc.SetString(5, '1/5')
        bc.SetString(6, '1/6')
        bc.SetString(8, '1/8')
        bc.SetString(16, '1/16')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionRenderSettings(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(0, 'Project Render Settings')
        bc.SetString(1, 'Viewport Render')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescriptionTarget(self, node, description, descId, name, idParent, valDefault):
        bc = c4d.BaseContainer()

        bc.SetString(0, 'Open in Picture Viewer')
        bc.SetString(1, 'Save')
        bc.SetString(2, 'Open in Picture Viewer and Save')

        return self.GetDDescriptionCreateCombo(node, description, descId, name, idParent, bc, valDefault=valDefault)


    def GetDDescription_CreateGroup(self, node, description, singleId, idGroup, idParent, nameGroup, numColumns=1, defaultOpen=True):
        descIdGroup = c4d.DescID(c4d.DescLevel(idGroup, c4d.DTYPE_GROUP, node.GetType()))
        if singleId is None or descIdGroup.IsPartOf(singleId)[0]:
            if not self.GetDDescriptionCreateGroup(description, descIdGroup, nameGroup, idParent, numColumns, defaultOpen):
                return False

        return True

    def GetDDescription_CreateCombo(self, node, description, singleId, descId, name, idParent, valDefault, funcCreate):
        if singleId is None or descId.IsPartOf(singleId)[0]:
            if not funcCreate(node, description, descId, name, idParent, valDefault):
                return False

        return True


    def GetDDescription_CreateWidget(self, node, description, singleId, idPV, idParent):
        COMBOS = { ID_PREF_THUMB_SIZE                   : self.GetDDescriptionCreateComboThumbSize,
                   ID_PREF_THUMB_ASPECT_RATIO           : self.GetDDescriptionCreateComboThumbAspectRatio,
                   ID_PREF_THUMB_UPDATES                : self.GetDDescriptionCreateComboThumbInterval,
                   ID_PREF_THUMB_MARKER_ID              : self.GetDDescriptionCreateComboThumbMarker,
                   ID_PREF_CREATE_CAMERAS               : self.GetDDescriptionCreateCreateCameras,
                   ID_PREF_PROOF_SHEET_SIZE_FACTOR      : self.GetDDescriptionProofSheetSize,
                   ID_PREF_PROOF_SHEET_RENDER_SETTINGS  : self.GetDDescriptionRenderSettings,
                   ID_PREF_PROOF_SHEET_TARGET           : self.GetDDescriptionTarget,
                   ID_PREF_RENDER_ALL_RENDER_SETTINGS   : self.GetDDescriptionRenderSettings,
                   ID_PREF_RENDER_ALL_TARGET            : self.GetDDescriptionTarget,
                   ID_PREF_IN_PALETTE_SIZE              : self.GetDDescriptionCreateComboThumbSize, # not exactly thumb size, but option names are the same...
                 }

        STATIC_TEXT = { ID_PREF_TEXT_IN_PALETTE,
                      }

        dataType = TABLE_PREFS[idPV][PV_DT]
        valDefault = TABLE_PREFS[idPV][PV_DEF]
        name = TABLE_PREFS[idPV][PV_NAME]

        descId = c4d.DescID(c4d.DescLevel(idPV, dataType, node.GetType()))

        if idPV in COMBOS:
            return self.GetDDescription_CreateCombo(node, description, singleId, descId, name, idParent, valDefault, COMBOS[idPV])

        if singleId is None or descId.IsPartOf(singleId)[0]:
            if dataType == c4d.DTYPE_BOOL:
                if not self.GetDDescriptionCreateCheckbox(node, description, descId, name, idParent, valDefault=valDefault):
                    return False

            elif dataType == c4d.DTYPE_LONG:
                if not self.GetDDescriptionCreateLong(description, descId, name, idParent, valDefault=valDefault):
                    return False

            elif dataType == c4d.DTYPE_COLOR:
                if not self.GetDDescriptionCreateColor(description, descId, name, idParent, valDefault=valDefault):
                    return False

            elif dataType == c4d.DTYPE_STRING:
                if idPV in STATIC_TEXT:
                    if not self.GetDDescriptionCreateString(description, descId, name, idParent, valDefault=valDefault, static=True):
                        return False

                else:
                    if not self.GetDDescriptionCreateString(description, descId, name, idParent, valDefault=valDefault):
                        return False

            elif dataType == c4d.DTYPE_BUTTON:
                if not self.GetDDescriptionCreateButton(description, descId, name, idParent):
                    return False

        return True


    def GetDDescription_GroupDesign(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_DESIGN, CAMSPREFERENCES_MAIN_GROUP, 'Design - Menu Bar'):
            return False

        PARAMS = [ ID_PREF_MENU_FILES_ENABLED,
                   ID_PREF_MENU_CAMERAS_ENABLED,
                   ID_PREF_PREV_NEXT_BUTTONS_ENABLED,
                   ID_PREF_DESIGN_BUTTONS_ENABLED,
                   ID_PREF_ALL_CAMS_BUTTON_ENABLED,
                   ID_PREF_UPDATE_THUMBS_BUTTON_ENABLED,
                   ID_PREF_SETTINGS_BUTTON_ENABLED,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_DESIGN):
                return False

        return True


    def GetDDescription_GroupDesignProject(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_DESIGN_CAMERA, CAMSPREFERENCES_MAIN_GROUP, 'Design - Project Selection'):
            return False

        PARAMS = [ ID_PREF_COMBO_ENABLED,
                   ID_PREF_THUMB_ENABLED,
                   ID_PREF_TABS_ENABLED,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_DESIGN_CAMERA):
                return False

        return True


    def GetDDescription_GroupDesignExtra(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_DESIGN_EXTRA, CAMSPREFERENCES_MAIN_GROUP, 'Design - Advanced Settings', defaultOpen=False):
            return False

        PARAMS = [ ID_PREF_LARGE_COMBO,
                   ID_PREF_POPUP_MODE,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_DESIGN_EXTRA):
                return False

        return True


    def GetDDescription_GroupThumbnails(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_THUMB, CAMSPREFERENCES_MAIN_GROUP, 'Thumbnails', defaultOpen=False):
            return False

        if c4d.GetC4DVersion() >= 22000:
            PARAMS = [ ID_PREF_THUMB_SIZE,
                       ID_PREF_THUMB_ASPECT_RATIO,
                       ID_PREF_THUMB_FILTER,
                       ID_PREF_THUMB_UPDATES,
                       ID_PREF_THUMB_TITLE,
                       ID_PREF_THUMB_OVERLAY_ICON,
                       ID_PREF_THUMB_OVERLAY_ANIM,
                     ]

        else:
            PARAMS = [ ID_PREF_THUMB_SIZE,
                       ID_PREF_THUMB_ASPECT_RATIO,
                       ID_PREF_THUMB_UPDATES,
                       ID_PREF_THUMB_TITLE,
                       ID_PREF_THUMB_OVERLAY_ICON,
                       ID_PREF_THUMB_OVERLAY_ANIM,
                     ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_THUMB):
                return False

        return True


    def GetDDescription_GroupActiveScene(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_ACTIVE_CAMERA, CAMSPREFERENCES_MAIN_GROUP, 'Thumbnail - Active Cam', defaultOpen=False):
            return False

        PARAMS = [ ID_PREF_THUMB_HIDE_ACTIVE,
                   ID_PREF_THUMB_FADE_ACTIVE,
                   ID_PREF_THUMB_MARKER_ID,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_ACTIVE_CAMERA):
                return False

        # Subgroup: Active Scene Overlay
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_THUMB_OVERLAY, ID_GROUP_PREF_ACTIVE_CAMERA, '', numColumns=2):
            return False

        PARAMS = [ ID_PREF_THUMB_OVERLAY_LT,
                   ID_PREF_THUMB_OVERLAY_RT,
                   ID_PREF_THUMB_OVERLAY_LB,
                   ID_PREF_THUMB_OVERLAY_RB,
                   ID_PREF_THUMB_OVERLAY_LABEL,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_THUMB_OVERLAY):
                return False

        # Subgroup: Active Scene Overlay Colors
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_OVERLAY_COLORS, ID_GROUP_PREF_ACTIVE_CAMERA, '', numColumns=2):
            return False

        PARAMS = [ ID_PREF_THUMB_MARKER_COLOR,
                   ID_PREF_THUMB_MARKER_LAYER_COLOR,
                   ID_PREF_THUMB_LABEL_COLOR,
                   ID_PREF_THUMB_LABEL_LAYER_COLOR,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_OVERLAY_COLORS):
                return False

        return True


    def GetDDescription_GroupCreateCamera(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_CREATE_CAMERAS, CAMSPREFERENCES_MAIN_GROUP, 'Create Cams'):
            return False

        PARAMS = [ ID_PREF_CREATE_CAMERAS,
                   ID_PREF_CREATE_C4D_CAMERAS_ACTIVATE,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_CREATE_CAMERAS):
                return False

        return True


    def GetDDescription_GroupProofSheet(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_PROOF_SHEET, CAMSPREFERENCES_MAIN_GROUP, 'Proof Sheet'):
            return False

        PARAMS = [ ID_PREF_PROOF_SHEET_SIZE_FACTOR,
                   ID_PREF_PROOF_SHEET_RENDER_SETTINGS,
                   ID_PREF_PROOF_SHEET_TARGET,
                   ID_PREF_PROOF_SHEET_OVERLAY_NAME,
                   ID_PREF_PROOF_SHEET_EDITOR_CAM,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_PROOF_SHEET):
                return False

        # Subgroup: Filename with Token button
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_PROOF_SHEET_FILENAME, ID_GROUP_PREF_PROOF_SHEET, '', numColumns=2):
            return False

        PARAMS = [ ID_PREF_PROOF_SHEET_FILENAME,
                   ID_PREF_PROOF_SHEET_FILENAME_TOKENS,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_PROOF_SHEET_FILENAME):
                return False

        return True


    def GetDDescription_GroupRenderAll(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_RENDER_ALL, CAMSPREFERENCES_MAIN_GROUP, 'Render Cams'):
            return False

        PARAMS = [ ID_PREF_RENDER_ALL_RENDER_SETTINGS,
                   ID_PREF_RENDER_ALL_TARGET,
                   ID_PREF_RENDER_ALL_EDITOR_CAM,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_RENDER_ALL):
                return False

        # Subgroup: Filename with Token button
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_RENDER_ALL_FILENAME, ID_GROUP_PREF_RENDER_ALL, '', numColumns=2):
            return False

        PARAMS = [ ID_PREF_RENDER_ALL_FILENAME,
                   ID_PREF_RENDER_ALL_FILENAME_TOKENS,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_RENDER_ALL_FILENAME):
                return False

        return True


    def GetDDescription_GroupInPaletteSwitcher(self, node, description, singleId):
        if not self.GetDDescription_CreateGroup(node, description, singleId, ID_GROUP_PREF_IN_PALETTE, CAMSPREFERENCES_MAIN_GROUP, 'In Palette Combo Switcher'):
            return False

        PARAMS = [ ID_PREF_TEXT_IN_PALETTE,
                   ID_PREF_IN_PALETTE_SIZE,
                 ]

        for idPV in PARAMS:
            if not self.GetDDescription_CreateWidget(node, description, singleId, idPV, ID_GROUP_PREF_IN_PALETTE):
                return False

        return True


    def GetDDescription(self, node, description, flags):
        if not description.LoadDescription('camspreferences'):
            print('{0} ERROR: Failed to load Description'.format(PLUGIN_NAME))
            return False

        if flags & c4d.DESCFLAGS_DESC_NEEDDEFAULTVALUE:
            self.Init(node, description)

        singleId = description.GetSingleDescID()

        if not self.GetDDescription_CreateWidget(node, description, singleId, ID_PREF_PLUGIN_ENABLED, CAMSPREFERENCES_MAIN_GROUP):
            return False

        if not self.GetDDescription_GroupDesign(node, description, singleId):
            return False

        if not self.GetDDescription_GroupDesignProject(node, description, singleId):
            return False

        if not self.GetDDescription_GroupDesignExtra(node, description, singleId):
            return False

        if not self.GetDDescription_GroupThumbnails(node, description, singleId):
            return False

        if not self.GetDDescription_GroupActiveScene(node, description, singleId):
            return False

        if not self.GetDDescription_GroupActiveScene(node, description, singleId):
            return False

        if not self.GetDDescription_GroupCreateCamera(node, description, singleId):
            return False

        if not self.GetDDescription_GroupProofSheet(node, description, singleId):
            return False

        if not self.GetDDescription_GroupRenderAll(node, description, singleId):
            return False

        if not self.GetDDescription_GroupInPaletteSwitcher(node, description, singleId):
            return False

        return True, flags | c4d.DESCFLAGS_DESC_LOADED


    def SetDParameter(self, node, id, data, flags):
        bc = GetWorldPrefs()

        idPV = id[0].id

        if idPV not in TABLE_PREFS:
            PrintError('Unknown Parameter: {0}'.format(idPV))
            return True

        TABLE_PREFS[idPV][PV_SET](bc, idPV, data)

        if idPV == ID_PREF_PLUGIN_ENABLED and data == False:
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_CLOSE_CAMS)

        NEED_EVENT_CHANGE_DESIGN = { ID_PREF_MENU_FILES_ENABLED,
                                     ID_PREF_MENU_CAMERAS_ENABLED,
                                     ID_PREF_COMBO_ENABLED,
                                     ID_PREF_LARGE_COMBO,
                                     ID_PREF_TABS_ENABLED,
                                     ID_PREF_THUMB_ENABLED,
                                     ID_PREF_THUMB_SIZE,
                                     ID_PREF_THUMB_ASPECT_RATIO,
                                     ID_PREF_THUMB_FILTER,
                                   }

        if idPV in NEED_EVENT_CHANGE_DESIGN:
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_CHANGE_DESIGN)

        NEED_EVENT_CHANGE_MENU_BUTTONS = { ID_PREF_PREV_NEXT_BUTTONS_ENABLED,
                                           ID_PREF_DESIGN_BUTTONS_ENABLED,
                                           ID_PREF_ALL_CAMS_BUTTON_ENABLED,
                                           ID_PREF_UPDATE_THUMBS_BUTTON_ENABLED,
                                           ID_PREF_SETTINGS_BUTTON_ENABLED,
                                         }

        if idPV in NEED_EVENT_CHANGE_MENU_BUTTONS:
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_CHANGE_MENU_BUTTONS)

        NEED_EVENT_REDRAW_THUMBS = { ID_PREF_THUMB_TITLE,
                                     ID_PREF_THUMB_HIDE_ACTIVE,
                                     #ID_PREF_THUMB_FADE_ACTIVE, # Event is send in MSG_DESCRIPTION_CHECKUPDATE, because otherwise we won't get a reliable event from slider changes
                                     ID_PREF_THUMB_MARKER_ID,
                                     ID_PREF_THUMB_MARKER_COLOR,
                                     ID_PREF_THUMB_MARKER_LAYER_COLOR,
                                     ID_PREF_THUMB_LABEL_COLOR,
                                     ID_PREF_THUMB_LABEL_LAYER_COLOR,
                                     ID_PREF_THUMB_OVERLAY_LT,
                                     ID_PREF_THUMB_OVERLAY_RT,
                                     ID_PREF_THUMB_OVERLAY_LB,
                                     ID_PREF_THUMB_OVERLAY_RB,
                                     ID_PREF_THUMB_OVERLAY_LABEL,
                                     ID_PREF_THUMB_OVERLAY_ICON,
                                     ID_PREF_THUMB_OVERLAY_ANIM,
                                   }

        if idPV in NEED_EVENT_REDRAW_THUMBS:
            c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_REDRAW_THUMBS)

        return True, flags | c4d.DESCFLAGS_SET_PARAM_SET


    def GetDParameter(self, node, id, flags):
        bc = GetWorldPrefs()

        idPV = id[0].id

        if idPV not in TABLE_PREFS:
            PrintError('Unknown Parameter: {0}'.format(idPV))
            return False

        return True, TABLE_PREFS[idPV][PV_GET](bc, idPV), flags | c4d.DESCFLAGS_GET_PARAM_GET


    def GetDEnabling(self, node, id, t_data, flags, itemdesc):
        pluginEnabled = ReadPrefValue(ID_PREF_PLUGIN_ENABLED)

        idPV = id[0].id

        if idPV not in TABLE_PREFS:
            PrintError('Unknown Parameter: {0}'.format(idPV))
            return True

        return pluginEnabled


    def CreateTokenPopup(self):
        bc = c4d.BaseContainer()

        tokens = c4d.modules.tokensystem.GetAllTokenEntries()

        for idxToken, t in enumerate(tokens):
            bc.SetString(c4d.FIRST_POPUP_ID + idxToken, '{0} - {1}'.format(t['_help'], t['_example']))

        return bc, tokens


    def CommandButtonTokens(self, id):
        bcMenu, tokens = self.CreateTokenPopup()

        result = c4d.gui.ShowPopupDialog(cd=None, bc=bcMenu, x=c4d.MOUSEPOS, y=c4d.MOUSEPOS)

        if result == 0:
            return

        idxToken = result - c4d.FIRST_POPUP_ID

        sToken = '$' + tokens[idxToken]['_token']

        bc = GetWorldPrefs()

        if id == ID_PREF_RENDER_ALL_FILENAME_TOKENS:
            idFilename = ID_PREF_RENDER_ALL_FILENAME

        else:
            idFilename = ID_PREF_PROOF_SHEET_FILENAME

        StorePrefValue(idFilename, ReadPrefValue(idFilename) + sToken)


    def Message(self, node, type, data):
        if type == c4d.MSG_DESCRIPTION_CHECKUPDATE:
            id = data['descid'][0].id

            if id == ID_PREF_THUMB_FADE_ACTIVE:
                c4d.SpecialEventAdd(PLUGIN_ID_CAMS_COREMESSAGE, p1=SUBID_COREMESSAGE_REDRAW_THUMBS)

        elif type == c4d.MSG_DESCRIPTION_COMMAND:
            id = data['id'][0].id

            if id == ID_PREF_RENDER_ALL_FILENAME_TOKENS:
                self.CommandButtonTokens(id)

            elif id == ID_PREF_PROOF_SHEET_FILENAME_TOKENS:
                self.CommandButtonTokens(id)

        return True
