import os, time
from xml.dom import minidom
import c4d
from cams_utils_c4d_version import *
from cams_ids import *
from cams_utils_image import *
from cams_utils_pref import *
from cams_dialog_question import *


def iterAllObjects(obj):
    while obj is not None:
        yield obj

        objNext = obj.GetNext()

        if obj.GetDown() is not None:
            for objChild in iterAllObjects(obj.GetDown()):
                yield objChild

        obj = objNext


def GetObjectListByType(doc=None, root=None, typeObj=c4d.Obase):
    if doc is None:
        doc = c4d.documents.GetActiveDocument()

    if root is None:
        root = doc.GetFirstObject()

    objList = []

    for obj in iterAllObjects(root):
        if obj.CheckType(typeObj):
            objList.append(obj)

    return objList


def ObjectGUID(obj):
    return obj.GetGUID() & 0x1FFFFFFFFFFFFFFFFFFFFFFF


def ObjectListToDict(objs):
    d = {}

    for obj in objs:
        d[ObjectGUID(obj)] = obj

    return d


def GetDocList():
    doc = c4d.documents.GetFirstDocument()

    listDocs = []

    while doc is not None:
        listDocs.append([doc, {}])

        doc = doc.GetNext()

    return listDocs


def GetDocIndex(listDocs, doc):
    for idxDoc, docTest, _ in enumarate(listDocs):
        if doc == docTest:
            return idxDoc

    PrintError('Doc not found!')
    return None


def GetDocThumbDict(listDocs, doc):
    for docTest, dictThumbs in listDocs:
        if not docTest.IsAlive():
            continue

        if doc == docTest:
            return dictThumbs

    dictThumbsNew = {}
    listDocs.append([doc, dictThumbsNew])

    return dictThumbsNew


def GarbageCollectDocThumbDict(dictCameras, dictThumbs):
    for guid, bmpThumb in list(dictThumbs.items()):
        if guid not in dictCameras:
            del dictThumbs[guid]


def GarbageCollectDocList(listDocs):
    listDocsCurrent = GetDocList()

    for docEntry in list(listDocs):
        doc = docEntry[0]

        found = False
        for docTest, _ in listDocsCurrent:
            if docTest == doc:
                found = True
                break

        if not found:
            listDocs.remove(docEntry)


def GetUpdatedDocList(listDocsOld=[]):
    listDocsCurrent = GetDocList()

    for doc, dictThumbs in listDocsOld:
        if not doc.IsAlive():
            continue

        for idxDoc, (docTest, _) in enumerate(listDocsCurrent):
            if docTest == doc:
                listDocsCurrent[idxDoc][1] = dictThumbs
                break

    return listDocsCurrent


def GetDefaultDocBC():
    bc = c4d.BaseContainer()

    bc.SetInt32(ID_DOC_NUM_CAMS, 0)
    bc.SetContainer(ID_DOC_CAMS, c4d.BaseContainer())

    return bc


def GetDocBC(doc=None):
    if doc is None:
        doc = c4d.documents.GetActiveDocument()

    bcDoc = doc.GetDataInstance().GetContainerInstance(PLUGIN_ID_CAMS)

    if bcDoc is None:
        doc.GetDataInstance().SetContainer(PLUGIN_ID_CAMS, GetDefaultDocBC())
        bcDoc = doc.GetDataInstance().GetContainerInstance(PLUGIN_ID_CAMS)

    return bcDoc


def CameraBCToDict(bcDoc):
    bcCamsL1 = bcDoc.GetContainerInstance(ID_DOC_CAMS)

    if bcCamsL1 is None:
        return {}

    dictCameras = {}

    for addrL1, bcCamsL2 in bcCamsL1:
        for addrL2, bcCamsL3 in bcCamsL2:
            for addrL3, bcCam in bcCamsL3:
                guid = AddrToGUID(addrL1, addrL2, addrL3)

                bcCamInst = bcCamsL1.GetContainerInstance(addrL1).GetContainerInstance(addrL2).GetContainerInstance(addrL3)

                objCam = bcCamInst.GetLink(ID_CAM_OBJ)
                flagsHDirty = bcCamInst.GetInt32(ID_CAM_LAST_HDIRTY, 0)
                isEditorCam = bcCamInst.GetBool(ID_CAM_IS_EDITOR, False)

                dictCameras[guid] = (objCam, flagsHDirty, bcCamInst, isEditorCam)

    return dictCameras


def CameraBCToList(bcDoc):
    bcCamsL1 = bcDoc.GetContainerInstance(ID_DOC_CAMS)

    if bcCamsL1 is None:
        return []

    camerasUnsorted = []

    objCamEditor = None
    flagsHDirtyEditor = None
    bcCamInstEditor = None
    guidEditor = None

    for addrL1, bcCamsL2 in bcCamsL1:
        for addrL2, bcCamsL3 in bcCamsL2:
            for addrL3, bcCam in bcCamsL3:
                guid = AddrToGUID(addrL1, addrL2, addrL3)

                bcCamInst = bcCamsL1.GetContainerInstance(addrL1).GetContainerInstance(addrL2).GetContainerInstance(addrL3)

                objCam = bcCam.GetLink(ID_CAM_OBJ)
                flagsHDirty = bcCam.GetInt32(ID_CAM_LAST_HDIRTY, 0)
                isEditorCam = bcCam.GetBool(ID_CAM_IS_EDITOR, False)

                if isEditorCam:
                    objCamEditor = objCam
                    flagsHDirtyEditor = flagsHDirty
                    bcCamInstEditor = bcCamInst
                    guidEditor = guid

                    continue

                camerasUnsorted.append((objCam.GetName(), (objCam, flagsHDirty, bcCamInst, guid, isEditorCam)))

    if len(camerasUnsorted) > 0:
        cameras = [c for _, c in sorted(camerasUnsorted, key=lambda cu: cu[0])]

    else:
        cameras = []

    cameras.append((objCamEditor, flagsHDirtyEditor, bcCamInstEditor, guidEditor, True))

    return cameras


def GUIDToAddr(guid):
    addrL1 = ((guid & 0x1FFFFFFFC000000000000000) >> 62)
    addrL2 = ((guid & 0x000000003FFFFFFF80000000) >> 31)
    addrL3 =   guid & 0x00000000000000007FFFFFFF

    return addrL1, addrL2, addrL3

def AddrToGUID(addrL1, addrL2, addrL3):
    return (addrL1 << 62) | (addrL2 << 31) | addrL3


def StoreCamera(bcDoc, objCam, isEditorCam=False):
    guid = objCam.GetGUID()

    dictCameras = CameraBCToDict(bcDoc)

    flagsHDirty = objCam.GetHDirty(HDIRTYFLAGS_OBJECT | HDIRTYFLAGS_OBJECT_MATRIX | HDIRTYFLAGS_TAG)

    if guid in dictCameras:
        flagsHDirtyOld = dictCameras[guid][1]

        return flagsHDirtyOld != flagsHDirty

    bcCamsL1 = bcDoc.GetContainerInstance(ID_DOC_CAMS)

    if bcCamsL1 is None:
        PrintError('Help! No Cam BC L1.')
        return False

    addrL1, addrL2, addrL3 = GUIDToAddr(guid)

    if bcCamsL1.FindIndex(addrL1)['index'] == -1:
        bcCamsL1.SetContainer(addrL1, c4d.BaseContainer())
        bcCamsL2 = bcCamsL1.GetContainerInstance(addrL1)

    else:
        bcCamsL2 = bcCamsL1.GetContainerInstance(addrL1)

    if bcCamsL2 is None:
        PrintError('Help! No Cam BC L2.')
        return False

    if bcCamsL2.FindIndex(addrL2)['index'] == -1:
        bcCamsL2.SetContainer(addrL2, c4d.BaseContainer())
        bcCamsL3 = bcCamsL2.GetContainerInstance(addrL2)

    else:
        bcCamsL3 = bcCamsL2.GetContainerInstance(addrL2)

    if bcCamsL3 is None:
        PrintError('Help! No Cam BC L3.')
        return False

    numCams = bcDoc.GetInt32(ID_DOC_NUM_CAMS, 0)

    isNewCam = False

    if bcCamsL3.FindIndex(addrL3)['index'] == -1:
        bcCamsL3.SetContainer(addrL3, c4d.BaseContainer())

        numCams += 1
        isNewCam = True

        bcDoc.SetInt32(ID_DOC_NUM_CAMS, numCams)

    bcCam = bcCamsL3.GetContainerInstance(addrL3)

    if objCam is not None:
        bcCam.SetLink(ID_CAM_OBJ, objCam)

    bcCam.SetBool(ID_CAM_IS_EDITOR, isEditorCam)

    flagsHDirtyOld = bcCam.GetInt32(ID_CAM_LAST_HDIRTY, 0)

    return isNewCam or (flagsHDirtyOld != flagsHDirty)


def GarbageCollectCameraBC(bcDoc, cameras):
    numCams = bcDoc.GetInt32(ID_DOC_NUM_CAMS, 0)
    numCamsOld = numCams

    dictCameras = ObjectListToDict(cameras)

    bcCamsL1_i = bcDoc.GetContainerInstance(ID_DOC_CAMS)

    for addrL1, bcCamsL2 in bcCamsL1_i:
        bcCamsL2_i = bcCamsL1_i.GetContainerInstance(addrL1)

        for addrL2, bcCamsL3 in bcCamsL2:
            bcCamsL3_i = bcCamsL2_i.GetContainerInstance(addrL2)


            for addrL3, objCam in bcCamsL3:
                guid = AddrToGUID(addrL1, addrL2, addrL3)

                if guid not in dictCameras:
                    bcCamsL3_i.RemoveData(addrL3)
                    numCams -= 1

            if len(bcCamsL3_i) == 0:
                bcCamsL2_i.RemoveData(addrL2)

        if len(bcCamsL2_i) == 0:
            bcCamsL1_i.RemoveData(addrL1)

    bcDoc.SetInt32(ID_DOC_NUM_CAMS, numCams)

    return numCams != numCamsOld


def GetCameraDirtyList(doc=None):
    bcDoc = GetDocBC(doc)

    dictCameras = CameraBCToDict(bcDoc)

    camerasDirty = []

    for objCam, flagsHDirtyLast, bcCam in dictCameras.values():
        flagsHDirty = 0

        if objCam is not None:
            flagsHDirty = objCam.GetHDirty(HDIRTYFLAGS_OBJECT | HDIRTYFLAGS_OBJECT_MATRIX | HDIRTYFLAGS_TAG)

        if flagsHDirty != flagsHDirtyLast:
            bcCam.SetInt32(ID_CAM_LAST_HDIRTY, flagsHDirty)

            camerasDirty.append(objCam)

    return camerasDirty


def GetCameraIndex(cameras, objCam):
    for idxCam, (objCamTest, _, _, _, _) in enumerate(cameras):
        if objCamTest == objCam:
            return idxCam

    return None


def CloneDocumentFocussed(doc, objFocus, cloneOnly=False):
    if cloneOnly or objFocus is None:
        docClone = doc.GetClone(COPYFLAGS_NONE)

        return docClone, None

    bcObj = objFocus.GetDataInstance()

    bcObj[PLUGIN_ID_CAMS] = PLUGIN_ID_CAMS

    docClone = doc.GetClone(COPYFLAGS_NONE)

    objRoot = docClone.GetFirstObject()

    objFocusClone = None

    for obj in iterAllObjects(objRoot):
        if obj.GetDataInstance().FindIndex(PLUGIN_ID_CAMS)['index'] != -1:
            objFocusClone = obj
            break

    if objFocusClone is None:
        PrintError('Focus object not found in cloned document.')

    bcObj.RemoveData(PLUGIN_ID_CAMS)

    return docClone, objFocusClone


def GetEditorCamera(doc=None):
    if doc is None:
        doc = c4d.documents.GetActiveDocument()

    bdActive = doc.GetRenderBaseDraw()

    objCamEditor = bdActive.GetEditorCamera()

    return objCamEditor


def GetActiveCamera(doc=None):
    if doc is None:
        doc = c4d.documents.GetActiveDocument()

    bdActive = doc.GetRenderBaseDraw()

    objCamActive = None
    if bdActive.HasCameraLink():
        objCamActive = bdActive.GetSceneCamera(doc)
        isEditorCam = False

    else:
        objCamActive = bdActive.GetEditorCamera()
        isEditorCam = True

    return objCamActive, isEditorCam


def ActivateNextCamera(doc, previous=False):
    objCamActive, isEditorCamActive = GetActiveCamera(doc)

    bcDoc = GetDocBC(doc)

    cameras = CameraBCToList(bcDoc)

    idxCamActive = GetCameraIndex(cameras, objCamActive)

    if idxCamActive is None:
        PrintError('INDEX NONE')
        objCamActiveNew = cameras[0][0]

    else:
        if previous:
            idxCamActive = (idxCamActive - 1) % len(cameras)

        else:
            idxCamActive = (idxCamActive + 1) % len(cameras)

        objCamActiveNew = cameras[idxCamActive][0]

    bdActive = doc.GetRenderBaseDraw()

    bdActive.SetSceneCamera(objCamActiveNew)

    c4d.EventAdd()


def SearchRenderData(doc, name):
    rd = doc.GetFirstRenderData()

    while rd is not None:
        if rd.GetName() == name:
            return rd

        rd = rd.GetNext()

    return None


def CreateDefaultRenderDataViewportRender(w=50, h=50, name='{0} VP Rendersettings'.format(PLUGIN_NAME), docToInsert=None):
    rd = c4d.documents.RenderData()

    if rd is None:
        PrintError('Failed to create render settings')
        return None

    rd.SetName(name)

    vp = c4d.BaseList2D(c4d.RDATA_RENDERENGINE_PREVIEWHARDWARE)

    if vp is None:
        PrintError('Failed to create Preview Hardware Render VideoPost')
        return None

    rd.InsertVideoPostLast(vp)

    vp[c4d.VP_PREVIEWHARDWARE_ENHANCEDOPENGL] = True
    vp[c4d.VP_PREVIEWHARDWARE_TRANSPARENCY] = True
    vp[c4d.VP_PREVIEWHARDWARE_SHADOW] = True
    vp[c4d.VP_PREVIEWHARDWARE_POSTEFFECT] = True
    vp[c4d.VP_PREVIEWHARDWARE_NOISE] = True

    if c4d.GetC4DVersion() >= 18000:
        vp[c4d.VP_PREVIEWHARDWARE_TESSELLATION] = False
        vp[c4d.VP_PREVIEWHARDWARE_SSAO] = True
        vp[c4d.VP_PREVIEWHARDWARE_REFLECTIONS] = True

    if c4d.GetC4DVersion() >= 19000:
        vp[c4d.VP_PREVIEWHARDWARE_DEPTHOFFIELD] = True

    vp[c4d.VP_PREVIEWHARDWARE_ANTIALIASING] = 8

    if ReadPrefValue(ID_PREF_THUMB_FILTER) and c4d.GetC4DVersion() >= 22000:
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_MULTIAXIS] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_OBJECTHANDLES] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_WORLDAXIS] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_HUD] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_BASEGRID] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_GRID] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_HANDLES] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_HORIZON] = False
        vp[c4d.VP_PREVIEWHARDWARE_DISPLAYFILTER_CAMERA] = False

    rd[c4d.RDATA_RENDERENGINE] = c4d.RDATA_RENDERENGINE_PREVIEWHARDWARE
    rd[c4d.RDATA_XRES] = w
    rd[c4d.RDATA_YRES] = h

    if docToInsert is not None:
        docToInsert.InsertRenderDataLast(rd)

    return rd


def GetThumbRenderData(doc, activate=True):
    rd = SearchRenderData(doc, NAME_THUMB_RENDERDATA)

    if rd is None:
        rd = CreateDefaultRenderDataViewportRender(name=NAME_THUMB_RENDERDATA, docToInsert=doc)

    rd[c4d.RDATA_XRES] = THUMB_RENDER_RESOLUTION[0]
    rd[c4d.RDATA_YRES] = THUMB_RENDER_RESOLUTION[1]

    if activate:
        doc.SetActiveRenderData(rd)

    return rd


def CreateDefaultRenderDataImageRender(w=50, h=50, name='{0} Rendersettings'.format(PLUGIN_NAME), docToInsert=None, alpha=False, fast=False):
    rd = c4d.documents.RenderData()

    if rd is None:
        PrintError('Failed to create render settings')
        return None

    rd.SetName(name)

    rd[c4d.RDATA_XRES] = w
    rd[c4d.RDATA_YRES] = h
    rd[c4d.RDATA_ALPHACHANNEL] = alpha
    rd[c4d.RDATA_TEXTUREERROR] = False

    if fast:
        rd[c4d.RDATA_THRESHOLD] = 0.05
        rd[c4d.RDATA_RAYDEPTH] = 10
        rd[c4d.RDATA_REFLECTIONDEPTH] = 3
        rd[c4d.RDATA_SHADOWDEPTH] = 10
        rd[c4d.RDATA_LOD] = 0.9
        rd[c4d.RDATA_ENABLESPD] = False
        rd[c4d.RDATA_SSS] = False

    if docToInsert is not None:
        docToInsert.InsertRenderDataLast(rd)

    return rd


def GetImageRenderData(doc, activate=True, forProofSheet=False):
    if forProofSheet:
        nameRD = NAME_PROOF_SHEET_RENDERDATA
        useRenderSettings = ReadPrefValue(ID_PREF_PROOF_SHEET_RENDER_SETTINGS)

    else:
        nameRD = NAME_RENDER_ALL_RENDERDATA
        useRenderSettings = ReadPrefValue(ID_PREF_RENDER_ALL_RENDER_SETTINGS)

    rdRender = SearchRenderData(doc, nameRD)

    if rdRender is None:
        rdCurrent = doc.GetActiveRenderData()

        if useRenderSettings == 0:
            rdRender = rdCurrent.GetClone()
            doc.InsertRenderDataLast(rdRender)

        elif useRenderSettings == 1:
            rdRender = CreateDefaultRenderDataViewportRender(w=rdCurrent[c4d.RDATA_XRES], h=rdCurrent[c4d.RDATA_YRES], name=nameRD, docToInsert=doc)

    if activate:
        doc.SetActiveRenderData(rdRender)

    return rdRender


def StoreViewportFilter(bd):
    bc = c4d.BaseContainer()

    bc[c4d.BASEDRAW_DISPLAYFILTER_MULTIAXIS] = bd[c4d.BASEDRAW_DISPLAYFILTER_MULTIAXIS]
    bc[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHANDLES] = bd[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHANDLES]
    bc[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHIGHLIGHTING] = bd[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHIGHLIGHTING]
    bc[c4d.BASEDRAW_DISPLAYFILTER_WORLDAXIS] = bd[c4d.BASEDRAW_DISPLAYFILTER_WORLDAXIS]
    bc[c4d.BASEDRAW_DISPLAYFILTER_HUD] = bd[c4d.BASEDRAW_DISPLAYFILTER_HUD]
    bc[c4d.BASEDRAW_DISPLAYFILTER_BASEGRID] = bd[c4d.BASEDRAW_DISPLAYFILTER_BASEGRID]
    bc[c4d.BASEDRAW_DISPLAYFILTER_GRID] = bd[c4d.BASEDRAW_DISPLAYFILTER_GRID]
    bc[c4d.BASEDRAW_DISPLAYFILTER_HANDLES] = bd[c4d.BASEDRAW_DISPLAYFILTER_HANDLES]

    return bc


def RestoreViewportFilter(bd, bc):
    bd[c4d.BASEDRAW_DISPLAYFILTER_MULTIAXIS] = bc[c4d.BASEDRAW_DISPLAYFILTER_MULTIAXIS]
    bd[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHANDLES] = bc[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHANDLES]
    bd[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHIGHLIGHTING] = bc[c4d.BASEDRAW_DISPLAYFILTER_OBJECTHIGHLIGHTING]
    bd[c4d.BASEDRAW_DISPLAYFILTER_WORLDAXIS] = bc[c4d.BASEDRAW_DISPLAYFILTER_WORLDAXIS]
    bd[c4d.BASEDRAW_DISPLAYFILTER_HUD] = bc[c4d.BASEDRAW_DISPLAYFILTER_HUD]
    bd[c4d.BASEDRAW_DISPLAYFILTER_BASEGRID] = bc[c4d.BASEDRAW_DISPLAYFILTER_BASEGRID]
    bd[c4d.BASEDRAW_DISPLAYFILTER_GRID] = bc[c4d.BASEDRAW_DISPLAYFILTER_GRID]
    bd[c4d.BASEDRAW_DISPLAYFILTER_HANDLES] = bc[c4d.BASEDRAW_DISPLAYFILTER_HANDLES]


def HaveRedshift():
    return c4d.plugins.FindPlugin(1036219, c4d.PLUGINTYPE_VIDEOPOST) is not None

def HaveCorona():
    return c4d.plugins.FindPlugin(1030480, c4d.PLUGINTYPE_VIDEOPOST) is not None

def HaveOctane():
    return c4d.plugins.FindPlugin(1029525, c4d.PLUGINTYPE_VIDEOPOST) is not None

def HaveVray():
    return c4d.plugins.FindPlugin(1053272, c4d.PLUGINTYPE_VIDEOPOST) is not None


def DetectProgressiveRender(doc=None):
    if doc is None:
        doc = c4d.documents.GetActiveDocument()

    rd = doc.GetActiveRenderData()

    idRenderEngine = rd[c4d.RDATA_RENDERENGINE]

    if idRenderEngine == c4d.RDATA_RENDERENGINE_STANDARD or idRenderEngine == c4d.RDATA_RENDERENGINE_PREVIEWHARDWARE:
        return False

    if idRenderEngine == 1036219: # Redshift 3  # TODO Andreas: symbol?
        # Does Redshift rendering always terminate??? # TODO Andfreas: yet, currently it looks like
        return False

    vpRenderEngine = None
    vp = rd.GetFirstVideoPost()

    while vp is not None:
        if vp.CheckType(idRenderEngine):
            vpRenderEngine = vp
            break

        vp = vp.GetNext()

    if vpRenderEngine is None:
        # A render engine without video post? Congrats!!! Kiss my shiny little ass!
        PrintError('DetectProgressiveRender WTF?!?!?! ID renderer: {0}'.format(idRenderEngine))
        return True # we will not render with this shit

    if idRenderEngine == 1023342: # Physical render  # TODO Andreas: symbol?
        if vpRenderEngine[c4d.VP_XMB_RAYTRACING_SAMPLER] == 2:
            return True # Physical Render in Progressive mode

    elif id == 1030480: # Corona
        timeLimit = vpRenderEngine[c4d.CORONA_SETTINGS_PROGRESSIVE_TIMELIMIT]
        dtLimit = timeLimit.GetDateTime()
        h = dtLimit.hour
        m = dtLimit.minute
        s = dtLimit.second
        if vpRenderEngine[c4d.CORONA_SETTINGS_PROGRESSIVE_NOISELIMIT] == 0.0 and \
           vpRenderEngine[c4d.CORONA_SETTINGS_PROGRESSIVE_PASSES] == 0 and \
           h == 0  and m == 0 and s == 0:
           return True

    elif id == 1029525: # Octane
        if vpRenderEngine[c4d.VP_TIME_LIMIT] == 0.0:
            return True

    elif id == 1053272: # Vray
        if vpRenderEngine[c4d.SETTINGSIMAGESAMPLER_TYPE] == 3 and vpRenderEngine[c4d.SETTINGSIMAGESAMPLER_PROGRESSIVE_MAXTIME] == 0.0:
            return True

    #elif idRenderEngine == 1036219: # Redshift 3  # TODO Andreas: symbol?
    #    if vpRenderEngine[c4d.REDSHIFT_RENDERER_RENDER_MODE_FINAL] == 0:
    #        return False # Redshift in progressive mode
    #
    #    else:
    #        print('REDSHIFT NON PROGRESSIVE')

    #elif id == MAXWELL:
    #elif id == URENDER:
    #elif id == CYCLES:
    #elif id == PRORENDER:

    else: # Unknown render engine always forbidden
        return True


def ShowProgressiveRenderError(useCustomDialog=False):
    # TODO Andreas: WHY DOES OUR MODAL DIALOG LOCK UP C4D, WHEN USED IN CoreMessage???
    # https://plugincafe.maxon.net/topic/13430/c4d-locks-up-when-opening-dialog-from-messagedata-coremessage
    if useCustomDialog:
        msgs = [ 'Current render settings will render indefinitely!',
                 '',
                 'Either change render settings or',
                 'change option for render settings in preferences.',
               ]
        OpenErrorDialog(msgs=msgs, saveDefault=True)

    else:
        sMsg = '{0} ERROR\n\n'.format(PLUGIN_NAME)
        sMsg += 'Current render settings will render indefinitely!\n\n'
        sMsg += 'Either change render settings or\n'
        sMsg += 'change option for render settings in preferences.'
        c4d.gui.MessageDialog(sMsg, c4d.GEMB_OK | c4d.GEMB_ICONEXCLAMATION)

    PrintWarn('Progressive render settings not supported.')


def GetCameraIcon(objCam):
    for idTag in CAMERA_TAGS:
        tag = objCam.GetTag(idTag)

        if tag is not None:
            return GetIconImage(tag)

    return GetIconImage(objCam)


def GetCameraTag(objCam):
    for idTag in CAMERA_TAGS:
        tag = objCam.GetTag(idTag)

        if tag is not None:
            return tag

    return None


def GetAllCameraTags(objCam):
    tags = []

    for idTag in CAMERA_TAGS:
        tag = objCam.GetTag(idTag)

        if tag is not None:
            tags.append(tag)

    return tags


def IsAnimated(bl2d):
    return bl2d is not None and bl2d.GetFirstCTrack() is not None


def ShowRenderInBackgroundInfo():
    msgs = [ 'Image rendering will happen in background.',
             'When done, rendered image(s) will be saved and/or,',
             'opened in Picture Viewer (depending on preferences option).',
           ]
    OpenInfoDialog(msgs=msgs, saveDefault=True)


def ShowRenderInProgressQuestion():
    msgs = [ 'Deferred Renderer is currently still rendering.',
           ]

    result = OpenQuestionDialog(msgs=msgs, buttonLabels=['Add Job(s) to Queue', 'Forget Previous Jobs', 'Cancel'],
                                wButton=c4d.gui.SizePix(120), idxFocus=0, idxAbort=2)

    return result
