import time, hashlib
from ctypes import pythonapi, c_void_p, py_object
import c4d
# NO CUSTOM MODULE IMPORTS HERE!!!

####################################################################################################
# Some C4D version dependent implementations
####################################################################################################
def AtMousePointer(dlg, x, y):
    if c4d.GetC4DVersion() // 1000 >= 20:
        return x, y

    if x == -1 or y == -1:
        xyDlgScreen = dlg.Local2Screen()
        xyDlgGlobal = dlg.Local2Global()

        bcMouse = c4d.BaseContainer()
        result = dlg.GetInputState(c4d.BFM_INPUT_MOUSE, 0, bcMouse)

        #print('Screen ', xyDlgScreen)
        #print('Global ', xyDlgGlobal)
        #print('Mouse  ', bcMouse[c4d.BFM_INPUT_X], bcMouse[c4d.BFM_INPUT_Y])

        if xyDlgGlobal['x'] == 0 and xyDlgGlobal['y'] == 0:
            x = bcMouse[c4d.BFM_INPUT_X] + xyDlgScreen['x']
            y = bcMouse[c4d.BFM_INPUT_Y] + xyDlgScreen['y']

        else:
            x = bcMouse[c4d.BFM_INPUT_X]
            y = bcMouse[c4d.BFM_INPUT_Y]

    return x, y


def Hash31(s):
    sha = hashlib.sha1()

    sha.update(s.encode('utf-8'))

    r = int(sha.hexdigest(), 16)
    r &= 0x7FFFFFFF

    return r


def GetCoreMessageParam23(msg, id=c4d.BFM_CORE_PAR1):
    vptr = msg.GetVoid(id)

    pythonapi.PyCapsule_GetPointer.restype = c_void_p
    pythonapi.PyCapsule_GetPointer.argtypes = [py_object]

    return pythonapi.PyCapsule_GetPointer(vptr, None)


def GetCoreMessageParamOld(msg, id=c4d.BFM_CORE_PAR1):
    vptr = msg.GetVoid(id)

    pythonapi.PyCObject_AsVoidPtr.restype = c_void_p
    pythonapi.PyCObject_AsVoidPtr.argtypes = [py_object]

    return pythonapi.PyCObject_AsVoidPtr(vptr)


# "Function Pointers"
if c4d.GetC4DVersion() // 1000 < 23:
    if c4d.GeGetCurrentOS() == c4d.OPERATINGSYSTEM_OSX:
        MyHash = Hash31

    else:
        MyHash = hash

    MyTime = time.clock
    GetCoreMessageParam = GetCoreMessageParamOld

else:
    MyHash = Hash31
    MyTime = time.process_time
    GetCoreMessageParam = GetCoreMessageParam23


####################################################################################################
# C4D version dependent constants
####################################################################################################
if c4d.GetC4DVersion() // 1000 > 19:
    BUILDFLAGS_NONE = c4d.BUILDFLAGS_NONE
    COPYFLAGS_NONE = c4d.COPYFLAGS_NONE
    BMP_SAVEBIT_NONE = c4d.SAVEBIT_NONE
    GETACTIVEOBJECTFLAGS_NONE = c4d.GETACTIVEOBJECTFLAGS_NONE
    DESCFLAGS_GET_NONE = c4d.DESCFLAGS_GET_NONE
    DESCFLAGS_SET_NONE = c4d.DESCFLAGS_SET_NONE
    DESCFLAGS_DESC_NONE = c4d.DESCFLAGS_DESC_NONE
    HDIRTYFLAGS_OBJECT = c4d.HDIRTYFLAGS_OBJECT  # Docs say, it came with R19???
    HDIRTYFLAGS_OBJECT_MATRIX = c4d.HDIRTYFLAGS_OBJECT_MATRIX  # Docs say, it came with R19???
    HDIRTYFLAGS_OBJECT_HIERARCHY = c4d.HDIRTYFLAGS_OBJECT_HIERARCHY  # Docs say, it came with R19???
    HDIRTYFLAGS_TAG = c4d.HDIRTYFLAGS_TAG  # Docs say, it came with R19???

else:
    BUILDFLAGS_NONE = c4d.BUILDFLAGS_0
    COPYFLAGS_NONE = c4d.COPYFLAGS_0
    BMP_SAVEBIT_NONE = c4d.SAVEBIT_0
    GETACTIVEOBJECTFLAGS_NONE = c4d.GETACTIVEOBJECTFLAGS_0
    DESCFLAGS_GET_NONE = c4d.DESCFLAGS_GET_0
    DESCFLAGS_SET_NONE = c4d.DESCFLAGS_SET_0
    DESCFLAGS_DESC_NONE = c4d.DESCFLAGS_DESC_0
    HDIRTYFLAGS_OBJECT = 0x0002  # Docs say, it came with R19???
    HDIRTYFLAGS_OBJECT_MATRIX = 0x0004  # Docs say, it came with R19???
    HDIRTYFLAGS_OBJECT_HIERARCHY = 0x0008  # Docs say, it came with R19???
    HDIRTYFLAGS_TAG = 0x0010  # Docs say, it came with R19???
