import c4d
from cams_ids import *
from cams_ids_dialogs import *
from cams_utils import *


def CommandVisibilityAll(id):
    offsetId = id - ID_EDITOR_VISIBILITY_ALL_DEFAULT
    if offsetId == 0 or offsetId == 3:
        valVisibility = 2

    elif offsetId == 1 or offsetId == 4: # on
        valVisibility = 0

    else:
        valVisibility = 1

    if offsetId < 3:
        idVisibility = c4d.ID_BASEOBJECT_VISIBILITY_EDITOR

    else:
        idVisibility = c4d.ID_BASEOBJECT_VISIBILITY_RENDER

    doc = c4d.documents.GetActiveDocument()

    bcDoc = GetDocBC(doc)

    cameras = CameraBCToList(bcDoc)

    doc.StartUndo()

    for objCam, _, _, _, isEditorCam in cameras:
        if isEditorCam:
            continue

        doc.AddUndo(c4d.UNDOTYPE_CHANGE, objCam)

        objCam[idVisibility] = valVisibility

    doc.EndUndo()

    c4d.EventAdd()


def CommandCreateC4DCamera(id):
    doc = c4d.documents.GetActiveDocument()

    createRendererCam = ReadPrefValue(ID_PREF_CREATE_CAMERAS)
    tagRenderer = None

    if createRendererCam == 1: # Redshift
        tagRenderer = c4d.BaseTag(1036760)

    elif createRendererCam == 2: # Corona
        tagRenderer = c4d.BaseTag(1032177)

    elif createRendererCam == 3: # Ocatne
        tagRenderer = c4d.BaseTag(1029524)

    elif createRendererCam == 4: # Vray
        tagRenderer = c4d.BaseTag(1053283)

    objCamActive, isEditorCam = GetActiveCamera(doc)

    objCameraNew = objCamActive.GetClone()
    if objCameraNew is None:
        return

    tags = GetAllCameraTags(objCameraNew)
    for tag in tags:
        tag.Remove()

    if isEditorCam:
        objCameraNew[c4d.CAMERAOBJECT_STEREO_MODE] = 0

    realActive = doc.GetRealActiveObject()

    if realActive is None:
        objActive, isMultiple = None, False

    else:
        objActive, isMultiple = realActive

    tagCameraType = None
    idTarget = None

    if id == ID_CREATE_CAMERA_TARGET:
        objCameraNew.SetName('Target Camera')
        tagCameraType = c4d.BaseTag(c4d.Ttargetexpression)
        idTarget = c4d.TARGETEXPRESSIONTAG_LINK

    elif id == ID_CREATE_CAMERA_STEREO:
        objCameraNew.SetName('Stereo Camera')
        objCameraNew[c4d.CAMERAOBJECT_STEREO_MODE] = 1

    elif id == ID_CREATE_CAMERA_MOTION:
        objCameraNew.SetName('Motion Camera')
        tagCameraType = c4d.BaseTag(c4d.Tmotioncam)
        idTarget = c4d.TMOTIONCAM_TARGET_A1

        tagCameraType[c4d.TMOTIONCAM_NMOTION_CAM_ROT_INTENSITY] = 0.0
        tagCameraType[c4d.TMOTIONCAM_NMOTION_CAM_POS_INTENSITY] = 0.0

    elif id == ID_CREATE_CAMERA_CRANE:
        objCameraNew.SetName('Camera Crane')
        tagCameraType = c4d.BaseTag(c4d.Tcrane)
        idTarget = c4d.TCRANE_HEAD_TARGETLINK

    if tagCameraType is not None:
        objCameraNew.InsertTag(tagCameraType)

    if tagRenderer is not None:
        objCameraNew.InsertTag(tagRenderer)

    doc.StartUndo()

    doc.InsertObject(objCameraNew)
    doc.AddUndo(c4d.UNDOTYPE_NEW, objCameraNew)

    if (isMultiple or objActive is None) and idTarget is not None:
        objTarget = c4d.BaseObject(c4d.Onull)
        objTarget.SetName('Camera Target')
        if c4d.GetC4DVersion() // 1000 > 22:
            objTarget[c4d.ID_BASELIST_ICON_COLORIZE_MODE] = 1
            objTarget[c4d.ID_BASELIST_ICON_COLOR] = c4d.Vector(1.0, 0.9, 0.4)

        if objActive is not None:
            objTarget.SetMg(objActive.GetMg())

        doc.InsertObject(objTarget)
        doc.AddUndo(c4d.UNDOTYPE_NEW, objTarget)

    else:
        objTarget = objActive

    if idTarget is not None:
        tagCameraType[idTarget] = objTarget

    if id == ID_CREATE_CAMERA_MOTION:
        objPath = c4d.BaseObject(c4d.Osplinecircle)
        objPath.SetName('Motion Camera Path')

        if objActive is not None:
            vPathPos = objActive.GetAbsPos()
            vPathPos.y = 0

        else:
            vPathPos = objTarget.GetAbsPos()

        objPath.SetAbsPos(vPathPos)

        objPath[c4d.PRIM_PLANE] = 2 # X/Z plane
        objPath[c4d.PRIM_CIRCLE_RADIUS] = 400.0

        doc.InsertObject(objPath)
        doc.AddUndo(c4d.UNDOTYPE_NEW, objPath)

        tagCameraType[c4d.TMOTIONCAM_SPLINE_A] = objPath

    doc.EndUndo()

    doc.SetActiveObject(objCameraNew)
    if tagCameraType is not None:
        doc.SetActiveTag(tagCameraType)

        c4d.gui.ActiveObjectManager_SetObject(c4d.ACTIVEOBJECTMODE_TAG, tagCameraType, c4d.ACTIVEOBJECTMANAGER_SETOBJECTS_OPEN, activepage=c4d.DescID())

    else:
        c4d.gui.ActiveObjectManager_SetObject(c4d.ACTIVEOBJECTMODE_OBJECT, objCameraNew, c4d.ACTIVEOBJECTMANAGER_SETOBJECTS_OPEN, activepage=c4d.DescID())

    if ReadPrefValue(ID_PREF_CREATE_C4D_CAMERAS_ACTIVATE):
        bdActive = doc.GetRenderBaseDraw()

        bdActive.SetSceneCamera(objCameraNew)

    c4d.EventAdd()


def CommandCreateCamera(id):
    # C4D Cams cameras
    if id == ID_CREATE_CAMERA_PREVIEW:
        c4d.CallCommand(PLUGIN_ID_CAMS_COMMAND_PREVIEW_CAMERA, COMMAND_PREVIEW_CAMERA_SUBID_TURNTABLE)

    elif id == ID_CREATE_CAMERA_PREVIEW_INSIDEOUT:
        c4d.CallCommand(PLUGIN_ID_CAMS_COMMAND_PREVIEW_CAMERA, COMMAND_PREVIEW_CAMERA_SUBID_LOOK_AROUND)

    # C4D cameras
    elif id >= ID_CREATE_CAMERA_STANDARD and id <= ID_CREATE_CAMERA_CRANE:
        CommandCreateC4DCamera(id)

    # Redshift cameras
    elif id == ID_CREATE_CAMERA_RS_STANDARD:
        c4d.CallCommand(1038677) # Standard

    elif id == ID_CREATE_CAMERA_RS_FISHEYE:
        c4d.CallCommand(1038678) # Fisheye

    elif id == ID_CREATE_CAMERA_RS_SPHERICAL:
        c4d.CallCommand(1038679) # Spherical

    elif id == ID_CREATE_CAMERA_RS_CYLINDRICAL:
        c4d.CallCommand(1038680) # Cylindrical

    elif id == ID_CREATE_CAMERA_RS_STEREO_SPHERICAL:
        c4d.CallCommand(1038681) # Stereo-Spherical

    # Octane cameras
    elif id == ID_CREATE_CAMERA_OC_STANDARD:
        c4d.CallCommand(1033865) # Octane Camera

    # Corona cameras
    elif id == ID_CREATE_CAMERA_CO_STANDARD:
        c4d.CallCommand(1032262) # Corona Camera

    # Vray cameras
    elif id == ID_CREATE_CAMERA_VR_PHYSICAL:
        c4d.CallCommand(1054777) # Vray Physical Camera

    else:
        PrintError('Oh noes! User requested an unsupported camera type ({0})! Shame on me!!! I am not worthy! I am not worthy!'.format(id))


def CommandCameraActivate(doc, objCam):
    bdActive = doc.GetRenderBaseDraw()

    bdActive.SetSceneCamera(objCam)

    c4d.EventAdd()


def CommandCameraOpenInAttributeManager(doc, objCam):
    doc.SetActiveObject(objCam)
    doc.GetActiveObject() # SDK docs say is needed

    c4d.gui.ActiveObjectManager_SetObject(c4d.ACTIVEOBJECTMODE_OBJECT, objCam, c4d.ACTIVEOBJECTMANAGER_SETOBJECTS_OPEN, activepage=c4d.DescID())

    c4d.EventAdd()


def CommandCameraTagOpenInAttributeManager(doc, objCam, idxTag):
    tagCam = GetAllCameraTags(objCam)[idxTag]

    doc.SetActiveTag(tagCam)

    c4d.gui.ActiveObjectManager_SetObject(c4d.ACTIVEOBJECTMODE_TAG, tagCam, c4d.ACTIVEOBJECTMANAGER_SETOBJECTS_OPEN, activepage=c4d.DescID())

    c4d.EventAdd()


def CommandCameraChangeVisibility(doc, idVisibility, objCam, visibility):
    doc.StartUndo()

    doc.AddUndo(c4d.UNDOTYPE_CHANGE, objCam)

    objCam[idVisibility] = visibility

    doc.EndUndo()

    c4d.EventAdd()


def CommandCameraDelete(doc, objCam):
    doc.StartUndo()

    doc.AddUndo(c4d.UNDOTYPE_DELETE, objCam)

    objCam.Remove()

    doc.EndUndo()

    c4d.EventAdd()
