import os
import c4d
from cams_ids import *
# NO OTHER CUSTOM MODULE IMPORTS HERE!!!


def GetIconImage(idIconOrBl2d, factor=None):
    if type(idIconOrBl2d) == int:
        icon = c4d.gui.GetIcon(idIconOrBl2d)

    else:
        icon = idIconOrBl2d.GetIcon()

    if icon is not None:
        bmpIcon = icon['bmp'].GetClonePart(icon['x'], icon['y'], icon['w'], icon['h'])

    else:
        bmpIcon = None
        PrintError('Icon not found ({0})'.format(idIconOrBl2d))

    if factor is not None:
        bmpIcon = ScaleImage(bmpIcon, factor=factor)

    return bmpIcon


def CheckRessourceImage(filename, subfolder='img', suffix='.png'):
    filename = filename + suffix
    imgPath = os.path.join(os.path.dirname(__file__), subfolder, filename)

    return os.path.exists(imgPath)


def LoadImage(filename, subfolder='img', suffix='.png', guiPixelRatio=1.0, factor=None):
    filename = filename + suffix
    imgPath = os.path.join(os.path.dirname(__file__), subfolder, filename)

    bmp = c4d.bitmaps.BaseBitmap()
    result = bmp.InitWith(imgPath)

    if result[0] != c4d.IMAGERESULT_OK:
        PrintError('Failed to load image: {0}'.format(filename))
        return None

    bmp.SetData(c4d.BASEBITMAP_DATA_GUIPIXELRATIO, guiPixelRatio)

    if factor is not None:
        bmp = ScaleImage(bmp, factor=factor)

    return bmp


def ScaleImage(bmp, w=100, h=100, factor=0.0):
    guiPixelRatio = bmp.GetData(c4d.BASEBITMAP_DATA_GUIPIXELRATIO, 1.0)

    if factor == 1.0:
        return bmp

    elif factor != 0.0:
        w, h = bmp.GetSize()

        w = int(w * factor)
        h = int(h * factor)

    bmpScaled = c4d.bitmaps.BaseBitmap()

    bmpScaled.Init(w, h)

    bmp.ScaleIt(bmpScaled, 256, True, False)

    bmpScaled.SetData(c4d.BASEBITMAP_DATA_GUIPIXELRATIO, guiPixelRatio)

    return bmpScaled


def BlendPixel(bmpTarget, bmpTargetAlpha, r, g, b, a, xTarget, yTarget):
    rTarget, gTarget, bTarget = bmpTarget.GetPixel(xTarget, yTarget)

    rTarget = int(rTarget * (1.0 - a)) + int(r * a)
    gTarget = int(gTarget * (1.0 - a)) + int(g * a)
    bTarget = int(bTarget * (1.0 - a)) + int(b * a)

    bmpTarget.SetPixel(xTarget, yTarget, rTarget, gTarget, bTarget)

    if bmpTargetAlpha is not None:
        aTarget = bmpTarget.GetAlphaPixel(bmpTargetAlpha, xTarget, yTarget)

        a255 = a * 255.0

        if aTarget < a255:
            bmpTarget.SetAlphaPixel(bmpTargetAlpha, xTarget, yTarget, a255)


def FadeImageToBackground(bmp, fade):
    if fade == 0:
        return

    fade = fade / 100.0

    colorBg = c4d.gui.GetGuiWorldColor(c4d.COLOR_BG) * 255.0
    rBg = int(colorBg.x)
    gBg = int(colorBg.y)
    bBg = int(colorBg.z)

    w, h = bmp.GetSize()

    for y in range(h):
        for x in range(w):
            BlendPixel(bmp, None, rBg, gBg, bBg, fade, x, y)


# TODO support alpha
def CopyImageIntoImage(bmpSource, bmpTarget, xTarget, yTarget):
    w, h = bmpSource.GetSize()

    for y in range(h):
        for x in range(w):
            r, g, b = bmpSource.GetPixel(x, y)

            bmpTarget.SetPixel(xTarget + x, yTarget + y, r, g, b)


def DrawOverlay(bmp, bmpOverlay, pos=-1, x=0, y=0, xOffs=0, yOffs=0,
                customColor=False, rCustom=0, gCustom=0, bCustom=0,
                customAlpha=False, aCustom=255):
    if bmp is None:
        return None

    bmpAlpha = bmp.GetInternalChannel()

    bmpOverlayAlpha = bmpOverlay.GetInternalChannel()

    wOverlay, hOverlay = bmpOverlay.GetSize()

    w, h = bmp.GetSize()

    if pos == 0:
        xOffset = xOffs
        yOffset = yOffs

    elif pos == 1:
        xOffset = w // 2 - wOverlay // 2
        yOffset = yOffs

    elif pos == 2:
        xOffset = w - xOffs - wOverlay
        yOffset = yOffs

    elif pos == 3:
        xOffset = w - xOffs - wOverlay
        yOffset = h // 2 - hOverlay // 2

    elif pos == 4:
        xOffset = w - xOffs - wOverlay
        yOffset = h - yOffs - hOverlay

    elif pos == 5:
        xOffset = w // 2 - wOverlay // 2
        yOffset = h - yOffs - hOverlay

    elif pos == 6:
        xOffset = xOffs
        yOffset = h - xOffs - hOverlay

    elif pos == 7:
        xOffset = xOffs
        yOffset = h // 2 - hOverlay // 2

    elif pos == 8:
        xOffset = w // 2 - wOverlay // 2
        yOffset = h // 2 - hOverlay // 2

    else:
        xOffset = x
        yOffset = y

    if customColor:
        def GetPixelCustomColor(x, y):
            return rCustom, gCustom, bCustom

        OverlayGetPixel = GetPixelCustomColor

    else:
        OverlayGetPixel = bmpOverlay.GetPixel

    if customAlpha:
        def GetAlphaPixelCustomColor(bmp, x, y):
            return aCustom

        OverlayGetAlphaPixel = GetAlphaPixelCustomColor

    else:
        OverlayGetAlphaPixel = bmpOverlay.GetAlphaPixel

    if bmpAlpha is not None:
        def SetAlphaPixelCond(bmpAlpha, xTarget, yTarget, aOverlay):
            a = bmp.GetAlphaPixel(bmpAlpha, xTarget, yTarget)

            if a < aOverlay:
                bmp.SetAlphaPixel(bmpAlpha, xTarget, yTarget, aOverlay)

    else:
        def SetAlphaPixelCond(bmpAlpha, xTarget, yTarget, aOverlay):
            pass

    for y in range(hOverlay):
        for x in range(wOverlay):
            xTarget = x + xOffset
            yTarget = y + yOffset

            rOverlay, gOverlay, bOverlay = OverlayGetPixel(x, y)

            r, g, b = bmp.GetPixel(xTarget, yTarget)

            aOverlay = OverlayGetAlphaPixel(bmpOverlayAlpha, x, y)
            aOverlay01 = aOverlay / 255.0
            aOverlay01Inv = 1.0 - aOverlay01

            rResult = int(r * aOverlay01Inv + rOverlay * aOverlay01)
            gResult = int(g * aOverlay01Inv + gOverlay * aOverlay01)
            bResult = int(b * aOverlay01Inv + bOverlay * aOverlay01)

            bmp.SetPixel(xTarget, yTarget, rResult, gResult, bResult)

            SetAlphaPixelCond(bmpAlpha, xTarget, yTarget, aOverlay)

    return bmp


def DrawTextOverlay(bmpPreview, sLabel='S', sizeFactor=4.0, bold=False, italic=False, r=200, g=100, b=50, x=16, y=0):
    cm = c4d.bitmaps.GeClipMap()

    bmpPreviewAlpha = bmpPreview.GetInternalChannel()
    if not cm.InitWithBitmap(bmpPreview, bmpPreviewAlpha):
        return bmpPreview

    w, h = cm.GetDim()

    cm.SetDrawMode(c4d.GE_CM_DRAWMODE_COPY, 0)

    bcFontDefault = c4d.bitmaps.GeClipMap.GetDefaultFont(c4d.GE_FONT_DEFAULT_SYSTEM)
    size = c4d.bitmaps.GeClipMap.GetFontSize(bcFontDefault, c4d.GE_FONT_SIZE_INTERNAL)

    if bold:
        bcFontDefault[502] = 700 # 400: normal, 700: bold

    if italic:
        bcFontDefault[503] = 1 # 0: normal, >0 italic

    #bcFontDefault[504] = 1 # 2 leads to "drop", special character???

    cm.BeginDraw()

    cm.SetFont(bcFontDefault, size * sizeFactor)

    cm.SetColor(r, g, b)

    cm.TextAt(x, y, sLabel)

    cm.EndDraw()

    bmpResult = cm.GetBitmap().GetClone()

    if bmpPreviewAlpha is not None:
        # API limitation: GeClipMap.GetBitmap() does not deliver alpha...
        bmpResult.AddChannel(True, False)
        bmpResultAlpha = bmpResult.GetInternalChannel()

        cm.BeginDraw()

        for y in range(h):
            for x in range(w):
                _, _, _, a = cm.GetPixelRGBA(x, y)
                bmpResult.SetAlphaPixel(bmpResultAlpha, x, y, a)

        cm.EndDraw()

    return bmpResult
