import c4d
from cams_utils_c4d_version import *
from cams_ids import *
from cams_utils_image import *


ID_PREF_A                                  = 101
ID_PREF_PLUGIN_ENABLED                     = 1000

ID_PREF_MENU_FILES_ENABLED                 = 1010
ID_PREF_MENU_CAMERAS_ENABLED               = 1011
ID_PREF_DESIGN_BUTTONS_ENABLED             = 1012
ID_PREF_PREV_NEXT_BUTTONS_ENABLED          = 1013
ID_PREF_TABS_ENABLED                       = 1014
ID_PREF_COMBO_ENABLED                      = 1015
ID_PREF_ALL_CAMS_BUTTON_ENABLED            = 1016
ID_PREF_THUMB_ENABLED                      = 1023

ID_PREF_POPUP_MODE                         = 1024

ID_PREF_UPDATE_THUMBS_BUTTON_ENABLED       = 1025
ID_PREF_SETTINGS_BUTTON_ENABLED            = 1026

ID_PREF_SHOW_ATTRIBUTES                    = 1030

ID_PREF_LARGE_COMBO                        = 1040
ID_PREF_IN_PALETTE_SIZE                    = 1041
ID_PREF_TEXT_IN_PALETTE                    = 1042

ID_PREF_THUMB_UPDATES                      = 1050
ID_PREF_THUMB_SIZE                         = 1051
ID_PREF_THUMB_TITLE                        = 1052
ID_PREF_THUMB_FILTER                       = 1053
ID_PREF_THUMB_ASPECT_RATIO                 = 1054

ID_PREF_THUMB_HIDE_ACTIVE                  = 1060
ID_PREF_THUMB_FADE_ACTIVE                  = 1061
ID_PREF_THUMB_OVERLAY_LT                   = 1062
ID_PREF_THUMB_OVERLAY_RT                   = 1063
ID_PREF_THUMB_OVERLAY_LB                   = 1064
ID_PREF_THUMB_OVERLAY_RB                   = 1065
ID_PREF_THUMB_OVERLAY_LABEL                = 1066
ID_PREF_THUMB_MARKER_ID                    = 1067
ID_PREF_THUMB_MARKER_COLOR                 = 1068
ID_PREF_THUMB_LABEL_COLOR                  = 1069
ID_PREF_THUMB_MARKER_LAYER_COLOR           = 1070
ID_PREF_THUMB_LABEL_LAYER_COLOR            = 1071
ID_PREF_THUMB_OVERLAY_ICON                 = 1072
ID_PREF_THUMB_OVERLAY_ANIM                 = 1073

ID_PREF_CREATE_CAMERAS                     = 1080
ID_PREF_CREATE_C4D_CAMERAS_ACTIVATE        = 1081

ID_PREF_PROOF_SHEET_SIZE_FACTOR            = 1090
ID_PREF_PROOF_SHEET_RENDER_SETTINGS        = 1091
ID_PREF_PROOF_SHEET_OVERLAY_NAME           = 1092
ID_PREF_PROOF_SHEET_EDITOR_CAM             = 1093
ID_PREF_PROOF_SHEET_FILENAME               = 1094
ID_PREF_PROOF_SHEET_TARGET                 = 1095
ID_PREF_PROOF_SHEET_FILENAME_TOKENS        = 1096

ID_PREF_RENDER_ALL_RENDER_SETTINGS         = 1100
ID_PREF_RENDER_ALL_TARGET                  = 1101
ID_PREF_RENDER_ALL_EDITOR_CAM              = 1102
ID_PREF_RENDER_ALL_FILENAME                = 1103
ID_PREF_RENDER_ALL_FILENAME_TOKENS         = 1104

ID_PREF_WEIGHTS                            = 2000

ID_PREF_REQUESTER_CHOICES                  = 10000


PV_DT   = 0
PV_DEF  = 1
PV_GET  = 2
PV_SET  = 3
PV_NAME = 4


def GetAvailableMarker(subfolder='img'):
    availableMarker = {}
    pathImg = os.path.join(os.path.dirname(__file__), subfolder)

    for filenameMarker in os.listdir(pathImg):
        if filenameMarker[:6] != 'marker' or (filenameMarker[-4:] != '.png' and filenameMarker[-4:] != '.jpg'):
            continue

        bmpMarker = LoadImage(filenameMarker, subfolder=subfolder, suffix='')
        idMarker = MyHash(filenameMarker)

        availableMarker[idMarker] = (filenameMarker, bmpMarker, idMarker)

    return availableMarker


def SetPrefValue(bcPref, idPref, val):
    typeVal = type(val)

    if typeVal == bool or typeVal == c4d.DTYPE_BOOL:
        bcPref.SetBool(idPref, val)

    elif typeVal == int or typeVal == c4d.DTYPE_LONG:
        bcPref.SetInt32(idPref, val)

    elif typeVal == float or typeVal == c4d.DTYPE_REAL:
        bcPref.SetFloat(idPref, val)

    elif typeVal == str or typeVal == c4d.DTYPE_STRING:
        bcPref.SetString(idPref, val)

    elif typeVal == c4d.Vector or typeVal == c4d.DTYPE_VECTOR or typeVal == c4d.DTYPE_COLOR:
        bcPref.SetVector(idPref, val)

    elif typeVal == c4d.Matrix or typeVal == c4d.DTYPE_MATRIX:
        bcPref.SetMatrix(idPref, val)

    elif typeVal == c4d.BaseContainer or typeVal == c4d.DTYPE_SUBCONTAINER:
        bcPref.SetContainer(idPref, val)

    elif typeVal == c4d.DTYPE_BUTTON:
        return

    else:
        PrintError('Unknown data type: ID: {0}, Type: {1}, {2}'.format(idPref, typeVal, val))


def StorePrefValue(idPref, val):
    bcWorldPrefs = GetWorldPrefs()

    TABLE_PREFS[idPref][PV_SET](bcWorldPrefs, idPref, val)

    c4d.plugins.SetWorldPluginData(PLUGIN_ID_CAMS, bcWorldPrefs, add=True)


def GetPrefValue(bcPref, idPref):
    typeVal = TABLE_PREFS[idPref][PV_DT]
    valDefault = TABLE_PREFS[idPref][PV_DEF]

    if typeVal == c4d.DTYPE_BOOL:
        return bcPref.GetBool(idPref, valDefault)

    elif typeVal == c4d.DTYPE_LONG:
        return bcPref.GetInt32(idPref, valDefault)

    elif typeVal == c4d.DTYPE_REAL:
        return bcPref.GetFloat(idPref, valDefault)

    elif typeVal == c4d.DTYPE_STRING:
        return bcPref.GetString(idPref, valDefault)

    elif typeVal == c4d.DTYPE_VECTOR or typeVal == c4d.DTYPE_COLOR:
        return bcPref.GetVector(idPref, valDefault)

    elif typeVal == c4d.DTYPE_MATRIX:
        return bcPref.GetMatrix(idPref, valDefault)

    elif typeVal == c4d.DTYPE_SUBCONTAINER:
        return bcPref.GetContainer(idPref)

    elif typeVal == c4d.DTYPE_BUTTON:
        return None

    else:
        PrintError('Unknown data type: ID: {0}, Type: {1}, {2}'.format(idPref, typeVal, valDefault))


def ReadPrefValue(idPref):
    bcWorldPrefs = GetWorldPrefs()
    return TABLE_PREFS[idPref][PV_GET](bcWorldPrefs, idPref)


def TogglePrefValue(idPref):
    valueNew = not ReadPrefValue(idPref)

    StorePrefValue(idPref, valueNew)

    return valueNew


def GetDefaultPrefContainer(): # TODO Andreas: shouldn't be needed, if prefdata defaults worked correctly (which most likely is an issue in this code...)
    bcDefaultPrefs = c4d.BaseContainer()

    if bcDefaultPrefs is None:
        PrintError('PrefContainer! Houston, we have got a problem!')
        return None

    for idPV, (_, valDefault, _, Set, _) in TABLE_PREFS.items():
        Set(bcDefaultPrefs, idPV, valDefault)

    return bcDefaultPrefs


def GetWorldPrefs():
    bcWorldPrefs = c4d.plugins.GetWorldPluginData(PLUGIN_ID_CAMS)

    if bcWorldPrefs is None:
        bcWorldPrefs = GetDefaultPrefContainer()
        c4d.plugins.SetWorldPluginData(PLUGIN_ID_CAMS, bcWorldPrefs, True)

    return bcWorldPrefs


def GetPrefsContainer(idBc):
    bcWorldPrefs = GetWorldPrefs()

    if bcWorldPrefs is None:
        return None

    bc = bcWorldPrefs.GetContainerInstance(idBc)

    if bc is None:
        bcWorldPrefs.SetContainer(idBc, c4d.BaseContainer())

        c4d.plugins.SetWorldPluginData(PLUGIN_ID_CAMS, bcWorldPrefs, add=True)
        bc = bcWorldPrefs.GetContainerInstance(idBc) # just to be sure we return the correct instance

    return bc


def SetPrefsContainer(idBc, bc):
    bcWorldPrefs = GetWorldPrefs()

    if bcWorldPrefs is None:
        return None

    bcWorldPrefs.SetContainer(idBc, bc)

    c4d.plugins.SetWorldPluginData(PLUGIN_ID_CAMS, bcWorldPrefs, add=True)


def GetPrefThumbModeRender(bcPref, idPref):
    return True
    # TODO Andreas: reenable if preview mode is usable (if ever...)
    bcWorldPrefs = GetWorldPrefs()
    return bcWorldPrefs.GetBool(BCID_BOOL_PREF_THUMB_MODE_RENDER, DEFAULT_THUMB_MODE_RENDER)


def GetPrefThumbFilter(bcPref, idPref):
    if c4d.GetC4DVersion() < 22000:
        return False

    return GetPrefValue(bcPref, idPref)

def SetPrefThumbFilter(bcPref, idPref, enabled):
    if c4d.GetC4DVersion() < 22000:
        enabled = False

    SetPrefValue(bcPref, idPref, enabled)


def GetPrefThumbActiveOverlay():
    overlay = 0

    if ReadPrefValue(ID_PREF_THUMB_OVERLAY_LT):
        overlay |= 0x01

    if ReadPrefValue(ID_PREF_THUMB_OVERLAY_RT):
        overlay |= 0x02

    if ReadPrefValue(ID_PREF_THUMB_OVERLAY_LB):
        overlay |= 0x04

    if ReadPrefValue(ID_PREF_THUMB_OVERLAY_RB):
        overlay |= 0x08

    if ReadPrefValue(ID_PREF_THUMB_OVERLAY_LABEL):
        overlay |= 0x10

    return overlay


def GetPrefThumbMarkerId():
    availableMarker = GetAvailableMarker()
    if len(availableMarker) <= 0:
        PrintError('No marker images!')
        return None

    idMarker = ReadPrefValue(ID_PREF_THUMB_MARKER_ID)

    if idMarker not in availableMarker:
        # configured marker image seems to be missing, we'll take first available instead
        PrintError('Configured marker image {0} not available!'.format(idMarker))

        for key in availableMarker.keys():
            idMarker = key
            break

        StorePrefValue(ID_PREF_THUMB_MARKER_ID, idMarker)

    return idMarker


def GetPrefThumbMarkerImage():
    availableMarker = GetAvailableMarker()
    idMarker = ReadPrefValue(ID_PREF_THUMB_MARKER_ID)

    if idMarker is None:
        return None

    return availableMarker[idMarker][1]

def GetPrefThumbMarkerName():
    availableMarker = GetAvailableMarker()
    idMarker = ReadPrefValue(ID_PREF_THUMB_MARKER_ID)

    if idMarker is None:
        return None

    return availableMarker[idMarker][0]


def GetDefaultWeightBC(showAttributes):
    bc = c4d.BaseContainer()

    if showAttributes == 1:
        bc.SetInt32(c4d.GROUPWEIGHTS_PERCENT_H_CNT, 2)
        bc.SetFloat(c4d.GROUPWEIGHTS_PERCENT_H_VAL + 0, 25.0)
        bc.SetFloat(c4d.GROUPWEIGHTS_PERCENT_H_VAL + 1, 75.0)

        bc.SetInt32(c4d.GROUPWEIGHTS_PERCENT_W_CNT, 1)
        bc.SetFloat(c4d.GROUPWEIGHTS_PERCENT_W_VAL + 0, 100.0)

    elif showAttributes == 2:
        bc.SetInt32(c4d.GROUPWEIGHTS_PERCENT_H_CNT, 1)
        bc.SetFloat(c4d.GROUPWEIGHTS_PERCENT_H_VAL + 0, 100.0)

        bc.SetInt32(c4d.GROUPWEIGHTS_PERCENT_W_CNT, 2)
        bc.SetFloat(c4d.GROUPWEIGHTS_PERCENT_W_VAL + 0, 25.0)
        bc.SetFloat(c4d.GROUPWEIGHTS_PERCENT_W_VAL + 1, 75.0)

    return bc


def RemovePrefs():
    bcWorld = c4d.GetWorldContainerInstance()

    bcPlugins = bcWorld.GetContainerInstance(c4d.WPREF_PLUGINS)

    bcPlugins.RemoveData(PLUGIN_ID_CAMS)


TEXT_IN_PALETTE = 'Widget will need to be dragged to palette for changes to appear (use "Customize Palettes...").'


TABLE_PREFS = { ID_PREF_A                            : (c4d.DTYPE_STRING,       '',                                   GetPrefValue,        SetPrefValue,        ''),
                ID_PREF_PLUGIN_ENABLED               : (c4d.DTYPE_BOOL,         DEFAULT_PLUGIN_ENABLED,               GetPrefValue,        SetPrefValue,        'Enable {0}'.format(PLUGIN_NAME)),
                ID_PREF_MENU_FILES_ENABLED           : (c4d.DTYPE_BOOL,         DEFAULT_MENU_FILES_ENABLED,           GetPrefValue,        SetPrefValue,        'Show Menu'),
                ID_PREF_MENU_CAMERAS_ENABLED         : (c4d.DTYPE_BOOL,         DEFAULT_MENU_CAMERAS_ENABLED,         GetPrefValue,        SetPrefValue,        'Show Cams'),
                ID_PREF_DESIGN_BUTTONS_ENABLED       : (c4d.DTYPE_BOOL,         DEFAULT_DESIGN_BUTTONS_ENABLED,       GetPrefValue,        SetPrefValue,        'Show Design Button (Restarts Dialog)'),
                ID_PREF_PREV_NEXT_BUTTONS_ENABLED    : (c4d.DTYPE_BOOL,         DEFAULT_PREV_NEXT_BUTTONS_ENABLED,    GetPrefValue,        SetPrefValue,        'Show Prev/Next Buttons (Restarts Dialog)'),
                ID_PREF_ALL_CAMS_BUTTON_ENABLED      : (c4d.DTYPE_BOOL,         DEFAULT_ALL_CAMS_BUTTON_ENABLED,      GetPrefValue,        SetPrefValue,        'Show Command Panel Button (Restarts Dialog)'),
                ID_PREF_SETTINGS_BUTTON_ENABLED      : (c4d.DTYPE_BOOL,         DEFAULT_SETTINGS_BUTTON_ENABLED,      GetPrefValue,        SetPrefValue,        'Show Settings Button (Restarts Dialog)'),
                ID_PREF_UPDATE_THUMBS_BUTTON_ENABLED : (c4d.DTYPE_BOOL,         DEFAULT_UPDATE_THUMBS_BUTTON_ENABLED, GetPrefValue,        SetPrefValue,        'Show Update Thumbnails Button (Restarts Dialog)'),
                ID_PREF_TABS_ENABLED                 : (c4d.DTYPE_BOOL,         DEFAULT_TABS_ENABLED,                 GetPrefValue,        SetPrefValue,        'Show Tabs'),
                ID_PREF_COMBO_ENABLED                : (c4d.DTYPE_BOOL,         DEFAULT_COMBO_ENABLED,                GetPrefValue,        SetPrefValue,        'Show Dropdown'),
                ID_PREF_THUMB_ENABLED                : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_ENABLED,                GetPrefValue,        SetPrefValue,        'Show Thumbnails'),
                ID_PREF_POPUP_MODE                   : (c4d.DTYPE_BOOL,         DEFAULT_POPUP_MODE,                   GetPrefValue,        SetPrefValue,        'Close Dialog After Cam Selection'),
                ID_PREF_LARGE_COMBO                  : (c4d.DTYPE_BOOL,         DEFAULT_LARGE_COMBO,                  GetPrefValue,        SetPrefValue,        'Large Dropdown'),
                ID_PREF_IN_PALETTE_SIZE              : (c4d.DTYPE_LONG,         DEFAULT_IN_PALETTE_SIZE,              GetPrefValue,        SetPrefValue,        'Size'),
                ID_PREF_THUMB_UPDATES                : (c4d.DTYPE_LONG,         DEFAULT_THUMB_UPDATES,                GetPrefValue,        SetPrefValue,        'Update every...'),
                ID_PREF_THUMB_SIZE                   : (c4d.DTYPE_LONG,         DEFAULT_THUMB_SIZE,                   GetPrefValue,        SetPrefValue,        'Size'),
                ID_PREF_THUMB_TITLE                  : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_TITLE,                  GetPrefValue,        SetPrefValue,        'Show Cam Name'),
                ID_PREF_THUMB_FILTER                 : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_FILTER,                 GetPrefThumbFilter,  SetPrefThumbFilter,  'Filter HUD, Axis, Handles,...'),
                ID_PREF_THUMB_ASPECT_RATIO           : (c4d.DTYPE_LONG,         DEFAULT_THUMB_ASPECT_RATIO,           GetPrefValue,        SetPrefValue,        'Aspect Ratio'),
                ID_PREF_THUMB_HIDE_ACTIVE            : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_HIDE_ACTIVE,            GetPrefValue,        SetPrefValue,        'Hide Active Cam'),
                ID_PREF_THUMB_FADE_ACTIVE            : (c4d.DTYPE_LONG,         DEFAULT_THUMB_FADE_ACTIVE,            GetPrefValue,        SetPrefValue,        'Fade Active Cam'),
                ID_PREF_THUMB_OVERLAY_LT             : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_LT,             GetPrefValue,        SetPrefValue,        'Marker Left/Top'),
                ID_PREF_THUMB_OVERLAY_RT             : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_RT,             GetPrefValue,        SetPrefValue,        'Marker Right/Top'),
                ID_PREF_THUMB_OVERLAY_LB             : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_LB,             GetPrefValue,        SetPrefValue,        'Marker Left/Bottom'),
                ID_PREF_THUMB_OVERLAY_RB             : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_RB,             GetPrefValue,        SetPrefValue,        'Marker Right/Bottom'),
                ID_PREF_THUMB_OVERLAY_LABEL          : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_LABEL,          GetPrefValue,        SetPrefValue,        'Active Label'),
                ID_PREF_THUMB_OVERLAY_ICON           : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_ICON,           GetPrefValue,        SetPrefValue,        'Camera Type Icon'),
                ID_PREF_THUMB_OVERLAY_ANIM           : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_OVERLAY_ANIM,           GetPrefValue,        SetPrefValue,        'Camera Animation Marker'),
                ID_PREF_THUMB_MARKER_ID              : (c4d.DTYPE_LONG,         DEFAULT_THUMB_MARKER_ID,              GetPrefValue,        SetPrefValue,        'Active Cam Marker'),
                ID_PREF_THUMB_MARKER_COLOR           : (c4d.DTYPE_COLOR,        DEFAULT_THUMB_MARKER_COLOR,           GetPrefValue,        SetPrefValue,        'Marker Color'),
                ID_PREF_THUMB_MARKER_LAYER_COLOR     : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_MARKER_LAYER_COLOR,     GetPrefValue,        SetPrefValue,        'Use Layer Color (if available)'),
                ID_PREF_THUMB_LABEL_COLOR            : (c4d.DTYPE_COLOR,        DEFAULT_THUMB_LABEL_COLOR,            GetPrefValue,        SetPrefValue,        'Label Color'),
                ID_PREF_THUMB_LABEL_LAYER_COLOR      : (c4d.DTYPE_BOOL,         DEFAULT_THUMB_LABEL_LAYER_COLOR,      GetPrefValue,        SetPrefValue,        'Use Layer Color (if available)'),
                ID_PREF_SHOW_ATTRIBUTES              : (c4d.DTYPE_LONG,         DEFAULT_SHOW_ATTRIBUTES,              GetPrefValue,        SetPrefValue,        'Show Attributes'),
                ID_PREF_CREATE_CAMERAS               : (c4d.DTYPE_LONG,         DEFAULT_CREATE_CAMERAS,               GetPrefValue,        SetPrefValue,        'Create Standard Cameras with Tag'),
                ID_PREF_CREATE_C4D_CAMERAS_ACTIVATE  : (c4d.DTYPE_BOOL,         DEFAULT_CREATE_C4D_CAMERAS_ACTIVATE,  GetPrefValue,        SetPrefValue,        'Activate Standard Cameras'),
                ID_PREF_PROOF_SHEET_SIZE_FACTOR      : (c4d.DTYPE_LONG,         DEFAULT_PROOF_SHEET_SIZE_FACTOR,      GetPrefValue,        SetPrefValue,        'Size Factor'),
                ID_PREF_PROOF_SHEET_RENDER_SETTINGS  : (c4d.DTYPE_LONG,         DEFAULT_PROOF_SHEET_RENDER_SETTINGS,  GetPrefValue,        SetPrefValue,        'Render Settings'),
                ID_PREF_PROOF_SHEET_TARGET           : (c4d.DTYPE_LONG,         DEFAULT_PROOF_SHEET_TARGET,           GetPrefValue,        SetPrefValue,        'After rendering'),
                ID_PREF_PROOF_SHEET_OVERLAY_NAME     : (c4d.DTYPE_BOOL,         DEFAULT_PROOF_SHEET_OVERLAY_NAME,     GetPrefValue,        SetPrefValue,        'Show Camera Name'),
                ID_PREF_PROOF_SHEET_EDITOR_CAM       : (c4d.DTYPE_BOOL,         DEFAULT_PROOF_SHEET_EDITOR_CAM,       GetPrefValue,        SetPrefValue,        'Include Editor Cam'),
                ID_PREF_PROOF_SHEET_FILENAME         : (c4d.DTYPE_STRING,       DEFAULT_PROOF_SHEET_FILENAME,         GetPrefValue,        SetPrefValue,        'Path'),
                ID_PREF_PROOF_SHEET_FILENAME_TOKENS  : (c4d.DTYPE_BUTTON,       None,                                 GetPrefValue,        SetPrefValue,        'v'),
                ID_PREF_RENDER_ALL_RENDER_SETTINGS   : (c4d.DTYPE_LONG,         DEFAULT_RENDER_ALL_RENDER_SETTINGS,   GetPrefValue,        SetPrefValue,        'Render Settings'),
                ID_PREF_RENDER_ALL_TARGET            : (c4d.DTYPE_LONG,         DEFAULT_RENDER_ALL_TARGET,            GetPrefValue,        SetPrefValue,        'After rendering'),
                ID_PREF_RENDER_ALL_EDITOR_CAM        : (c4d.DTYPE_BOOL,         DEFAULT_RENDER_ALL_EDITOR_CAM,        GetPrefValue,        SetPrefValue,        'Include Editor Cam'),
                ID_PREF_RENDER_ALL_FILENAME          : (c4d.DTYPE_STRING,       DEFAULT_RENDER_ALL_FILENAME,          GetPrefValue,        SetPrefValue,        'Path'),
                ID_PREF_RENDER_ALL_FILENAME_TOKENS   : (c4d.DTYPE_BUTTON,       None,                                 GetPrefValue,        SetPrefValue,        'v'),
                ID_PREF_WEIGHTS                      : (c4d.DTYPE_SUBCONTAINER, c4d.BaseContainer(),                  GetPrefValue,        SetPrefValue,        ''),
                ID_PREF_TEXT_IN_PALETTE              : (c4d.DTYPE_STRING,       TEXT_IN_PALETTE,                      GetPrefValue,        SetPrefValue,        ''),
              }
